/********* a very simple ADS1115 ADC reader ***********
 *   by Ewald Kendziorra sr, 2019 www.kendziorra.nl    *
 *******************************************************
 *        default configured for 3.3V modules!!!       *
 *        ======================================       *
 *          resolution 0.000125V (by VCC 3.3V)         *
 *  gain amplifier mode: 001 = 4.096V in 32768 steps   *
 *  number off steps, if VCC is exactly 3.3V: 26400    *
 *  volt per step: 0.000125V  (26400 * 0.000125 = 3.3) *
 *  mode: SINGLE-ENDED (4 to 16 A-ports)               *
 *  maximum analog input: VCC + 0.3V                   *
 *******************************************************
 *  USING (up to 4 modules each 4, port 0-15):         *
 ***  uint16_t steps = getAdc(portnr)                  *
 ***  double volt = volt per step * steps              *
 *******************************************************/

// look in this filelocation for: CalcNTCCalibratorNTCcalculator.htm, ThermistorCalc.png, thermistors.png
 
#include <sADS1115.h>

const uint8_t port = 0;  // the ADS1115 (addr)port (0x48)0-3, (0x49)4-7, (0x4A)8-11, (0x4B)12-15 
double vcc = 3.283;      // measured VCC on ADS1115 (you have to measure it by yourself)

void setup() {
  Serial.begin(115200);
  Serial.println("\n...\n");
  Wire.begin(SDA, SCL);  // ESP32: gpio21, gpio22
  sInitAds1115(vcc);
  printf("port 0 - maxStepsADS1115: %d, voltStepADS1115: %0.6f\n", maxStepsADS1115, voltStepADS1115);
}

void loop() {
  uint16_t adc = sAds1115(port);  // the measured ADC value
  double volt = voltStepADS1115 * adc;     // the volt value (voltStepADS1115 * measured ADC value)
  printf("Volt: %0.3f, steps: %d \n", volt, adc);
  delay(5000);
}
