/**  sBlobNVS.h: A very small and simple to use NVS_FLASH with STRUCTUREs / ARRAYs  ** 
 **  Totally free to use and modify. Ewald  Kendziorra sr, 2020, www.kendziorra.nl  **
 **  -------------------------------------------------------------------------------**/
#include <sBlobNVS.h>

///////////////////// BEGIN BLOBs NVS values /////////////////////
// We prefer to use arrays for BLOB storage in NVS FLASH (formerly EEPROM)
/** change the 'enum' names and array as desired **
 ** declare quantity and max. field width        **
 **         ========          ===========        **/
enum   /*typeU8*/    {U8name1, U8name2, U8name3, U8name4, U8name5};
uint8_t  typeU8[5] = {     1,        2,       3,       4,       5};
enum   /*typeFL*/    {FLname1, FLname2, FLname3};
float    typeFL[3] = {    1.1,     2.2,     3.3};
enum   /*typeCH*/    {CHname1, CHname2, CHname3, CHname4};
char typeCH[4][21] = { "abcd",  "efgh", "ijkl",   "mnop"};
char hallo[61] = "BLOB (Binary Large OBobject). NVS (Non Volatile Storage)";
int8_t test = 25;     // (unsigned)int can be used like a array with '(void*)&'
double dub = 123.45;  // float and double can be used like a array with '(void*)&' 
struct {char name[21] = "my name"; uint8_t nr = 42; bool test = false;} my_struct;
////////////////////// END BLOBs NVS values //////////////////////

bool basicBlobs();                   // clear all BLOBs, save the actual values
bool writeBlobs(bool mode = false);  // write the actual in use values to BLOB
bool readBlobs(bool mode = false);   // read the last saved values from BLOB

void setup() {
  Serial.begin(115200);
  while (!Serial); delay(1000);
  Serial.println();
  bool ok = true;
  bool firsttime = false;
  ////////
  // ok = basicBlobs(); firsttime = true; // uncomment to format NVS and write the standard values
  ////////
  if (ok) ok = readBlobs();
  if (!ok) {
    if (basicBlobs()) ok = readBlobs();
    firsttime = true; 
  }
  if (!ok) Serial.println("BLOBs problem, now we using the initiation data!");
  Serial.println("=========================================");
  if (firsttime) Serial.println("Clean start, we use the declarated values!\n");
  else Serial.println("ESP32 (re)START, we use values from BLOBs!\n");
  entriesBLOB();  // write used space to 'nvs_stats'
  Serial.printf("Used space total: %u, free: %u, used: %u\n", nvs_stats.total_entries, nvs_stats.free_entries, nvs_stats.used_entries);
  Serial.println("An anthology of used data:");
  Serial.printf("       U8name3: %u       U8name5: %u\n", typeU8[U8name3], typeU8[U8name5]);  // we use the enum names ;=)
  Serial.printf("       FLname2: %.2f\n", typeFL[FLname2]);  // we use the enum names ;=)
  Serial.printf("       CHname4: %s\n", typeCH[CHname4]);    // we use the enum names ;=)
  Serial.printf("         hallo: %s\n", hallo);
  Serial.printf("          test: %d\n", test);
  Serial.printf("           dub: %010.4f\n", dub);
  Serial.printf("my_struct.name: %s\n", my_struct.name);
  if (firsttime) {
    typeU8[U8name3] = 123;
    typeU8[U8name5] = 27;
    typeFL[FLname2] = 654.32;
    strcpy(typeCH[CHname4], "changed");
    strcpy(hallo, "BLOBs is still easy to use!");
    strcpy(my_struct.name, "Ewald");
    test = -56;
    dub = 2.5432;
    if (!writeBlobs()) Serial.println("BLOBs problem, data NOT saved!");
    else Serial.println("Comment 'ok = basicBlobs()' and reStart ESP32 to see the changed!");
  }
}

void loop() {
  // bool ok;
  // ok = readBlobs();      // use this to read the last saved values
  // ok = readBlobs(true);  // use this to read the original values
  // ok = writeBlobs();     // use this to save the current values
}

bool basicBlobs() {  // use this only as first step in setup() to save and using the original values
  initBLOB();  // erase all Flash data 
  bool ok = writeBlobs(true);  // write the original values as BLOBs
  if (ok) ok = writeBlobs();   // write the work values as BLOBs
  return ok;
}

// BLOB (Binary Large OBobject) so it expects composite objects (string, array, buffer,...)
bool writeBlobs(bool mode) {  // (optional)true: write BLOBs to standard values
  bool ok = openBLOB();
  if (ok) ok = writeBLOB((mode ? "__typeU8" : "typeU8"), typeU8, sizeof(typeU8));
  if (ok) ok = writeBLOB((mode ? "__typeFL" : "typeFL"), typeFL, sizeof(typeFL));
  if (ok) ok = writeBLOB((mode ? "__typeCH" : "typeCH"), typeCH, sizeof(typeCH));
  if (ok) ok = writeBLOB((mode ? "__hallo" : "hallo"), hallo, sizeof(hallo));
  if (ok) ok = writeBLOB((mode ? "__dub" : "dub"), (void*)&dub, sizeof(dub));      // a single number value 
  if (ok) ok = writeBLOB((mode ? "__test" : "test"),(void*)&test, sizeof(test));   // a single number value      
  if (ok) {  // a structure
    uint8_t structLen = sizeof(my_struct), structBuf[structLen];
    memcpy(&structBuf, &my_struct, structLen);
    ok = writeBLOB((mode ? "__my_struct" : "my_struct"), structBuf, structLen);
  }
  closeBLOB();
  return ok;
}

bool readBlobs(bool mode) {  // (optional)true: read BLOBs from standard values
  bool ok = openBLOB();
  if (ok) ok = readBLOB((mode ? "__typeU8" : "typeU8"), typeU8, sizeof(typeU8));
  if (ok) ok = readBLOB((mode ? "__typeFL" : "typeFL"), typeFL, sizeof(typeFL));
  if (ok) ok = readBLOB((mode ? "__typeCH" : "typeCH"), typeCH, sizeof(typeCH));
  if (ok) ok = readBLOB((mode ? "__hallo" : "hallo"), hallo, sizeof(hallo));
  if (ok) ok = readBLOB((mode ? "__dub" : "dub"), (void*)&dub, sizeof(dub));      // a single number value   
  if (ok) ok = readBLOB((mode ? "__test" : "test"), (void*)&test, sizeof(test));  // a single number value 
  if (ok) {  // a structure
    uint8_t structLen = sizeof(my_struct), structBuf[structLen];
    ok = readBLOB((mode ? "__my_struct" : "my_struct"), structBuf, structLen);
    if (ok) memcpy(&my_struct, &structBuf, structLen);
  }
  closeBLOB();
  return ok;  
}
