/**  sBlobNVS.h: A very small and simple to use NVS_FLASH with STRUCTUREs / ARRAYs  ** 
 **  Totally free to use and modify. Ewald  Kendziorra sr, 2020, www.kendziorra.nl       **
 **  ----------------------------------------------------------------------------------  **/
#include <sBlobNVS.h>  // a very simple and small but useful alternative to "Preferences.h"
// We only use BLOB (Binary Large OBobject) in the Flash memory (same memory as FAT and SPIFFS).
// With BLOB we can simple write and read byte and character streams.
// What about numerical values? In BLOB we convert it simple to and from a chararter string.
// In this way we only use one write and read instead of ten different storage functions.
// It is like files in a directory but very simple without bells and whistles.
// - naam: fixed as "my_namespace"(naam is Namespace like a DIR name, max 15 chars)
// - key: free choice (like a File name, max 15 chars)
// 'multiple char arrays' and 'structures' are also easy to handle with BLOB.
//==================
// byte stream: values, arrays or structures as a memory block using the memcpy function
//   struct data; / data (array)
//   size_t len = sizeof(data); 
//   byte v[len];
// write to BLOB:
//   memcpy(v, &data, len);
//   writeBLOB(key, v, len);
// read from BLOB:
//   readBLOB(key, v, len);
//   memcpy(&data, v, len);
/**
  void   initBLOB();     // use this only to erase all Flash data (as format a drive, partition table)
  size_t usedBLOB();     // count UsedEntries, entry in flash
  size_t freeBLOB();     // count FreeEntries
  size_t totalBLOB();    // count AllEntries
  bool   openBLOB();     // open Namespace (open file)
  void   closeBLOB();    // close Namespace (close file)
  bool   clearBLOB();    // remove all keys in the used Namespace (delete all files in a dir)
  bool   removeBLOB(const char* key);  // remove one key in the used Namespace (delete one file in a dir)
  size_t existBLOB(const char* key);   // key exist? 0 or byte length (file exist?)
  size_t writeBLOB(const char* key, const void* value, size_t len);  // write a byte stream (write binary)
  size_t readBLOB(const char* key, void* buf, size_t maxLen);        // read  a byte stream (read binary)
**/

// ##### example with char array ##### //
         /** field labels **/
const char fldLamp1[8][10] = {"key",   "status", "mode", "hourOn", "minuteOn", "hourOff", "minuteOff", "offsetOn"},
         /** original field values **/ // valOrgLamp1[0] is a CONST key
         valOrgLamp1[8][6] = {"lamp1", "0",      "1",    "7",      "30",       "23",      "15",        "10"};
char valLamp1[8][6] = {""};  /** field values in use (in/out NvsFlash) **/
const size_t itemsLamp1 = 8,
             lenLamp1 = sizeof(valLamp1);
// ##### example with structure ##### //
struct { 
  const char name[5] = "key",
           status[7] = "status",
             mode[5] = "mode",
           hourOn[7] = "hourOn",
         minuteOn[9] = "minuteOn",
          hourOff[8] = "hourOff",
       minuteOff[10] = "minuteOff",
         offsetOn[9] = "offsetOn";
} fldLamp2;  /** field labels **/
struct {  /** original field values **/
  const char name[6] = "lamp2";  // fixed name/key
  byte    status =  0;  // 0:off, 1:on
  byte      mode =  1;  // 0:stop, 1:morning, 2:evening
  byte    hourOn =  7;  // morning on
  byte  minuteOn = 30;
  byte   hourOff = 23;  // evening off
  byte minuteOff = 15;
  byte sunOffset = 10;  // sur rise/set offset
} valOrgLamp2,  /** original field values **/
  valLamp2;     /** field values in use (in/out NvsFlash) **/
const size_t itemsLamp2 = 8,
             lenLamp2 = sizeof(valLamp2);

// how to save the BASIC STRUCTURE/CHAR ARRAY values
void useOrg();  // save the original INIT values
// how to save the WORK STRUCTURE/CHAR ARRAY values
void putAll();  // save the changed values
// how to read the WORK STRUCTURE/CHAR ARRAY values
void getAll();  // read the last used/saved valus
void printLamp1();
void printLamp2();

void setup() {
  Serial.begin(115200);
  while (!Serial); delay(1000);
  Serial.println("\n");

  bool firstime = false;
//  initBLOB();  // unComment this to erase all Flash data

  byte v[(lenLamp1 > lenLamp2 ? lenLamp1 : lenLamp2)]; 
  Serial.printf("openBLOB: %u and test if data exist...\n", openBLOB());  // open Namespace
  if (existBLOB(valOrgLamp1[0]) != lenLamp1 || existBLOB(valOrgLamp2.name) != lenLamp2) { 
    initBLOB();  // erase all and use the orginal values
    Serial.printf("Count: UsedEntries = %d, FreeEntries = %d, AllEntries = %d\n", usedBLOB(), freeBLOB(), totalBLOB());
    Serial.printf("openBLOB: %u\n", openBLOB());
    Serial.printf("after reset freeEntries %u\n", freeBLOB());
    useOrg();  // original values to flash and field values in use
    firstime = true;
  }
  else {
    Serial.printf("\tData exist!\n\tNew start with al data exist in Flash!\n\t--------------------------------------\n"); 
    getAll();
  }

  printLamp1();  // char array valLamp1
  printLamp2();  // structure  valLamp2
  if (firstime) {
    Serial.printf("\t======================================\n"
                  "\tchange values valLamp1 and/or valLamp2\n"
                  "\t   call putAll() and restart ESP32\n"
                  "\t======================================\n");
    Serial.println("changing valLamp1[1] and valLamp2.status");
    strcpy(valLamp1[1], "1");  // lamp1:   char value 'status'
    valLamp2.status = 2;       // lamp2: struct value 'status'
    putAll();
  }
  Serial.printf("Count: UsedEntries = %d, FreeEntries = %d, AllEntries = %d\n", usedBLOB(), freeBLOB(), totalBLOB());
  closeBLOB();
}

void loop() {
  // put your main code here, to run repeatedly:
}

// how to save the BASIC STRUCTURE/CHAR ARRAY values
void useOrg() {  // save the original INIT values
  byte v[(lenLamp1 > lenLamp2 ? lenLamp1 : lenLamp2)];
  // (char array)
  memcpy(v, &valOrgLamp1, lenLamp1);
  memcpy(&valLamp1, v, lenLamp1);  // and copy to valLamp1 char array
  writeBLOB(valOrgLamp1[0], v, lenLamp1);
  // (structure)
  memcpy(v, &valOrgLamp2, lenLamp2);
  memcpy(&valLamp2, v, lenLamp2);  // and copy to valLamp2 structure
  writeBLOB(valOrgLamp2.name, v, lenLamp2);
}
// how to save the WORK STRUCTURE/CHAR ARRAY values
void putAll() {  // save the changed values
  byte v[(lenLamp1 > lenLamp2 ? lenLamp1 : lenLamp2)]; 
  // (char array)
  memcpy(v, &valLamp1, lenLamp1);
  writeBLOB(valLamp1[0], v, lenLamp1);
  // (structure)
  memcpy(v, &valLamp2, lenLamp2);
  writeBLOB(valLamp2.name, v, lenLamp2);
}
// how to read the WORK STRUCTURE/CHAR ARRAY values
void getAll() {  // read the last used/saved valus
  byte v[(lenLamp1 > lenLamp2 ? lenLamp1 : lenLamp2)]; 
  // (char array)
  readBLOB(valOrgLamp1[0], v, lenLamp1);
  memcpy(&valLamp1, v, lenLamp1);
  // (structure)
  readBLOB(valOrgLamp2.name, v, lenLamp2);
  memcpy(&valLamp2, v, lenLamp2);
}

void printLamp1() {
  Serial.printf("(char array) key: %s, len: %u\n", valOrgLamp1[0], lenLamp1);
  for (byte i = 0; i < itemsLamp1; i++) {
    Serial.printf("%10s: %s\n", fldLamp1[i], valLamp1[i]);
  }
}

void printLamp2() {
  Serial.printf("(structure) key: %s, len: %u\n", valOrgLamp2.name, lenLamp2);
  Serial.printf("%10s: %s\n", fldLamp2.name, valLamp2.name);
  Serial.printf("%10s: %u\n", fldLamp2.status, valLamp2.status);
  Serial.printf("%10s: %u\n", fldLamp2.mode, valLamp2.mode);
  Serial.printf("%10s: %u\n", fldLamp2.hourOn, valLamp2.hourOn);
  Serial.printf("%10s: %u\n", fldLamp2.minuteOn, valLamp2.minuteOn);
  Serial.printf("%10s: %u\n", fldLamp2.hourOff, valLamp2.hourOff);
  Serial.printf("%10s: %u\n", fldLamp2.minuteOff, valLamp2.minuteOff);
  Serial.printf("%10s: %u\n", fldLamp2.offsetOn, valLamp2.sunOffset);
}
