/**  sWifiFull: a very easy to use ESP32 WiFi library.
  *  - Automatic initialization and connection
  *  - 'stays alive' control
  *  - eleven freely selectable options
  *  Totally free to use and modify
  *  Ewald Kendziorra sr, 2020, www.kendziorra.nl 
  ****************************************************/ 

#define USE_STA       // use WiFi mode STA
#ifdef USE_STA        // only station mode
//  #define USE_STAIP   // if using a static STA IP (is needed with: USE_WS)
//  #define USE_STADNS  // use WiFi STA DNS
//  #define USE_OTA     // ArduinoOTA
//  #define USE_STACHAN  // used in AP_STA mode
  #define USE_TIME    // NTP, date, time, timezone, DST/Summertime /** my simpleLIB library: sDateTimeSun.h       **/
//  #define USE_WSER       // Web Server /** library: https://github.com/me-no-dev/ESPAsyncWebServer and https://github.com/me-no-dev/AsyncTCP **/
//  #define USE_WS       // WebSockets Server         /**   // used library: https://github.com/me-no-dev/ESPAsyncWebServer and https://github.com/me-no-dev/AsyncTCP **/
//  #define USE_WSCL     // ESP32 Websocket client (sSimpleLib)
//  #define USE_WTHR     // Weather  /** my simpleLIB library: sOpenweathermap.h **/
//  #define USE_SOLARMETEO  // KNMI SOLAR weersverwachting /** sSolarMeteo.h **/
#endif
//#define USE_AP        // use WiFi mode AP
#ifdef USE_AP         // only softAp mode
  #define USE_APIP    // if using a static AP IP 
  #define USE_NOW     // ESP-NOW only in softAP mode
  #define USE_APHID    // SSID hidden
  #define USE_APMAXCON // max connections, default 4, max 10
  #define USE_APCHAN  // used in AP_STA mode
#endif
#if defined(USE_NOW) || defined(USE_WS)
//  #define USE_PARSER   // parse the received message to fieldnames an fieldValues 
#endif
#if defined(USE_STA) && defined(USE_AP) 
  #define USE_LR      // ESP32 softAP Long Range mode up to 1000m (only in AP_STA mode)
#endif

const char staSsid[] = "EDIT_EDIT"@,
       staPassword[] = "EDIT_EDIT"@;
//const uint8_t staChannel = 11;
// ***** List of timezones https://sites.google.com/a/usapiens.com/opnode/time-zones
char timeZone[] = "CET-1CEST-2,M3.5.0/02:00:00,M10.5.0/03:00:00";  // EU Amsterdam/Barcelona with Summertime (DST)
const time_t syncTime = 12 * 3600;  // NTP time sync interval in sec. if 0 then NOT used
double latSun = 52.1234,
       lonSun = 5.5678;

#include <sWiFiFull.h>  // always under the optional #define xyz declarations 

void showIt(time_t t, bool gmt) {
  Serial.printf(" now: %02u-%02u-%04u %02u:%02u:%02u\n", day(t), month(t), year(t), hour(t), minute(t), second(t));
  sRiseSet rs = sSun(t, latSun, lonSun, gmt);  // (gmt) GMT/UTC time: true, local time: false
//  sRiseSet rs = sSun(t, latSun, lonSun);     // local time: default
  Serial.printf("rise: %02u:%02u,  set: %02u:%02u\n", hour(rs.rise), minute(rs.rise), hour(rs.set), minute(rs.set));
}

void setup() {
  Serial.begin(115200);
  delay(1000);
  Serial.println("Serial OK\n");
  setupWiFi();
  time_t t = now();
  showIt(t, false);  // local time today 
  t += 2 * sOneday;
  showIt(t, true);  // GMT/UTC time the day after tomorrow
}

void loop() {
//  loopWiFi();
}
