/**  sWifiFull: a very easy to use ESP32 WiFi library.
  *  - Automatic initialization and connection
  *  - 'stays alive' control
  *  - eleven freely selectable options
  *  Totally free to use and modify
  *  Ewald Kendziorra sr, 2020, www.kendziorra.nl 
  ****************************************************/ 

#define USE_STA        // use WiFi mode STA
#ifdef USE_STA         // only station mode
//  #define USE_STAIP    // if using a static STA IP (is needed with: USE_WS)
//  #define USE_STADNS   // use WiFi STA DNS
//  #define USE_STACHAN  // used in AP_STA mode
//  #define USE_OTA      // ArduinoOTA
  #define USE_TIME     // NTP, date, time, timezone, DST/Summertime /** my simpleLIB library: sDateTimeSun.h       **/
//  #define USE_WSER       // Web Server /** library: https://github.com/me-no-dev/ESPAsyncWebServer and https://github.com/me-no-dev/AsyncTCP **/
//  #define USE_WS       // WebSockets Server         /**   // used library: https://github.com/me-no-dev/ESPAsyncWebServer and https://github.com/me-no-dev/AsyncTCP **/
//  #define USE_WSCL     // ESP32 Websocket client (sSimpleLib)
//  #define USE_WTHR     // Weather  /** my simpleLIB library: sOpenweathermap.h **/
//  #define USE_SOLARMETEO  // KNMI SOLAR weersverwachting /** sSolarMeteo.h **/
#endif
//#define USE_AP         // use WiFi mode AP
#ifdef USE_AP          // only softAp mode
  #define USE_APIP     // if using a static AP IP 
  #define USE_APCHAN   // used in AP_STA mode
  #define USE_APHID    // SSID hidden
  #define USE_APMAXCON // max connections, default 4, max 10
  #define USE_NOW      // ESP-NOW only in softAP mode
#endif
#if defined(USE_NOW) || defined(USE_WS)
//  #define USE_PARSER   // parse the received message to fieldnames an fieldValues 
#endif
#if defined(USE_STA) && defined(USE_AP) 
  #define USE_LR       // ESP32 softAP Long Range mode up to 1000m (only in AP_STA mode)
#endif

const char staSsid[] = "EDIT_EDIT"@,
       staPassword[] = "EDIT_EDIT"@;
// ***** List of timezones https://sites.google.com/a/usapiens.com/opnode/time-zones
char timeZone[] = "CET-1CEST-2,M3.5.0/02:00:00,M10.5.0/03:00:00";  // EU Amsterdam/Barcelona with Summertime (DST)
const time_t syncTime = 12 * 3600;  // NTP time sync interval in sec. if 0 then NOT used
double latSun = 52.1234,
       lonSun = 5.5678;

#include <sWiFiFull.h>  // always under the optional #define xyz declarations 

sEpoch ep;     // UNIX epoch ep.sec and  milliseconds in ep.msec
sDateTime dt;  // local/GMT time and date values structure
void timeGet();

void setup() {
  Serial.begin(115200);
  while (!Serial); delay(1000);
  Serial.println("\n");
  setupWiFi();  // is necessary on this place - init all your defined choices 
  timeGet();  // sDateTimeSun.h, use this in your code where necessary
}

void loop() {
  loopWiFi();  // is necessary on this place - connection 'stays alive' control
  static time_t s = 0;  
  ep = sNow();
  if (s != ep.sec && (ep.sec % 60) == 0) {  // 60 sec print, sTimestampStr is useful with WebSocket communication
    Serial.printf("The current local time is: %s", ctime(&ep.sec));
    Serial.printf("\tsec: %u, msec: %u, ms-timestamp String: %s\n", ep.sec, ep.msec, sTimestampStr(ep).c_str());
    s = ep.sec;
  }
  yield();
}

void timeGet() {
  //  time: print as examples
  bool gmt = false;  // true: UTC/GMT values
  //sRiseSet rs = sSun( now(), lat, lon, gmt);
  sRiseSet rs = sSun(now(), latSun, lonSun);
  dt = sTimeDate(gmt);
  Serial.printf("SunRise: %02u:%02u, SunSet: %02u:%02u\n", hour(rs.rise), minute(rs.rise), hour(rs.set), minute(rs.set));
  time_t t = now();  // or use now(value): value is a + or - offset to the actual local time
  Serial.printf("sDateTimeSun.h - time_t t = now(),\n"
                "day(t), month(t), year(t),\n"
                "hour(t), minute(t), second(t),\n"
                "weekday(t), yweek(t), yday(t):\n\t"
                "%02u-%02u-%04u %02u:%02u:%02u local time "
                "weekday: %02u, yweek: %u, yday: %03u\n", 
                day(t), month(t), year(t),
                hour(t), minute(t), second(t), 
                weekday(t), yweek(t), yday(t));  // without the value (t) in the functions.... the time goes on
                // on a case-by-case basis you can also use these functions without value (t)
  //  dt-structure: print as example
  Serial.printf("sDateTimeSun.h dt-structure, time with milliseconds:\n\t"
                "%02d:%02d:%02d.%03d %s %02d %s (%02d) %02d  "
                "wday:%2d, week:%3d, yday:%4d, ",
                 dt.hour, dt.minute, dt.second, dt.msec,         // time with milliseconds
                 dt.dayS, dt.day, dt.monthS, dt.month, dt.year,  // month, day and year
                 dt.wday, dt.week, dt.yday);                     // diverse
  if (!gmt) Serial.printf("DST/Summertime: %s\n", (dt.dst > 0 ? "Y" : "N"));  // DST
  else Serial.println("GMT");
}
