﻿/**  sWifiFull: a very easy to use ESP32 WiFi library.
  *  - Automatic initialization and connection
  *  - 'stays alive' control
  *  - eleven freely selectable options
  *  Totally free to use and modify
  *  Ewald Kendziorra sr, 2020, www.kendziorra.nl 
  ****************************************************/ 

#define USE_STA        // use WiFi mode STA
#ifdef USE_STA         // only station mode
//  #define USE_STAIP    // if using a static STA IP (is needed with: USE_WS)
//  #define USE_STADNS   // use WiFi STA DNS
//  #define USE_OTA      // ArduinoOTA
//  #define USE_STACHAN  // used in AP_STA mode
  #define USE_TIME     // NTP, date, time, timezone, DST/Summertime /** my simpleLIB library: sDateTimeSun.h      **/
//  #define USE_WSER       // Web Server /** library: https://github.com/me-no-dev/ESPAsyncWebServer and https://github.com/me-no-dev/AsyncTCP **/
//  #define USE_WS       // WebSockets Server         /**   // used library: https://github.com/me-no-dev/ESPAsyncWebServer and https://github.com/me-no-dev/AsyncTCP **/
//  #define USE_WSCL     // ESP32 Websocket client (sSimpleLib)
//  #define USE_WTHR     // Weather  /** my simpleLIB library: sOpenweathermap.h **/
//  #define USE_SOLARMETEO  // KNMI SOLAR weersverwachting /** sSolarMeteo.h **/
#endif
//#define USE_AP         // use WiFi mode AP
#ifdef USE_AP          // only softAp mode
  #define USE_APIP     // if using a static AP IP 
  #define USE_NOW      // ESP-NOW only in softAP mode
  #define USE_APHID    // SSID hidden
  #define USE_APMAXCON // max connections, default 4, max 10
  #define USE_APCHAN   // used in AP_STA mode
#endif
#if defined(USE_NOW) || defined(USE_WS)
//  #define USE_PARSER   // parse the received message to fieldnames an fieldValues 
#endif
#if defined(USE_STA) && defined(USE_AP) 
  #define USE_LR       // ESP32 softAP Long Range mode up to 1000m (only in AP_STA mode)
#endif

/** or simple for this project only ...
  #define USE_STA      // use WiFi mode STA
  #define USE_TIME     // NTP, date, time, timezone, DST/Summertime my simpleLIB library: sDateTimeSun.h
**/

#include <M5StickC.h>   //   0.96 inch, 80*160 Colorful TFT LCD, ST7735S

const char *staSsid = "edit"@,       // universal ESP WiFi (auto re)connection ...
       *staPassword = "edit"@;  // ...(un)comment options in function
// ***** List of timezones https://sites.google.com/a/usapiens.com/opnode/time-zones
char timeZone[] = "CET-1CEST-2,M3.5.0/02:00:00,M10.5.0/03:00:00";  // EU Amsterdam/Barcelona with Summertime (DST)
                    /** "EST5EDT,M3.2.0,M11.1.0";// USA Eastern Time with DST **/
                    /** "CET-1";// EU Amsterdam/Barcelona NO DST/Summertime) **/
const time_t syncTime = 12 * 3600;  // NTP time sync interval in sec. if 0 then NOT used
#include <sWiFiFull.h>  // simpleLib  -  always under the above declarations 

/** DEFAULT VALUES in "sDateTimeSun.h" **/
// ***** strftime: http://www.cplusplus.com/reference/ctime/strftime
// ***** we assume by default that Monday is the first day of the week (ISO 8601)
//const char strfweek = 'V';// strftime: 'V'=ISO 8601 week number (01-53)
//const char strfwday = 'u';// strftime: 'u'=ISO 8601 weekday as number with Monday as 1 (1-7)
                  // ***** do not change the char[][] dimensions
//const char monthsLstr[12][21] = {"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"};
//const char monthsSstr[12][11] = {"Jan", "Feb", "Mar", "Apr", "May", "June", "July", "Aug", "Sept", "Oct", "Nov", "Dec"};
//const char daysLstr[7][21] = {"Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"};// strftime _wday: 'u'
//const char daysSstr[7][11] = {"Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"};// strftime _wday: 'u'
//** END DEFAULT VALUES **/

char buf[21];
uint8_t xpos;

hw_timer_t * timerSec = NULL;

#define GFXFF 1
#define FSS18 &FreeSans24pt7b
#define FSS9 &FreeSans9pt7b

void setup() {
  /** optional: use the default values or set your one 
   *  DEFAULT VALUES
   *  strfweek: 'V',  // week number     - strftime DEFAULT VALUE: 'V'=ISO 8601 week number (01-53)   
   *  strfwday: 'u';// day of the week - strftime DEFAULT VALUE: 'u'=ISO 8601 weekday as number with Monday as 1 (1-7)
   *  monthsLstr, monthsSstr, daysLstr, daysSstr: local Long/Short month an day names
   **/
//  sSetWeek(strfweek);
//  sSetWday(strfwday);
//  sSetMonthDayStr(monthsLstr, monthsSstr, daysLstr, daysSstr);
  /** optional END **/  
  M5.begin();
  M5.Lcd.setRotation(3);
  M5.Lcd.fillScreen(BLACK);
  M5.Lcd.setTextColor(TFT_YELLOW);
  M5.Lcd.setFreeFont(FSS9);
  M5.Lcd.print("\nsTime clock\nkendziorra.nl\nGet NTP-time...");
  M5.Lcd.setTextDatum(TC_DATUM);
  M5.Lcd.setTextColor(DARKGREEN);
  xpos = M5.Lcd.width() / 2 - 1;// Half the screen width
  setupWiFi();  // is necessary on this place - init all your defined choices 
}

void loop() {
  static uint8_t saveSecond, saveMinute, saveDay;
  static char colon[] = ":";
  loopWiFi();  // is necessary on this place - connection 'stays alive' control
  sEpoch ep = sNow(); // UNIX epoch ep.sec and  milliseconds in ep.msec
  sDateTime dt = sTimeDate(false, ep);// local time and date values
  if (saveSecond != dt.second) {
    // update date, day and week
    if (dt.day != saveDay) {
      M5.Lcd.fillScreen(BLACK);
      M5.Lcd.setFreeFont(FSS9);
      sprintf(buf, "%s %02u-%02u-%02u\n", dt.dayS, dt.day, dt.month, dt.year);
      M5.Lcd.drawString(buf, xpos, 45, GFXFF);
      sprintf(buf, "day %u   week %u", dt.wday, dt.week);
      M5.Lcd.drawString(buf, xpos, 63, GFXFF);
      saveDay = dt.day;
    }
    // update hh:mm
    M5.Lcd.setFreeFont(FSS18);
    if (saveMinute != dt.minute) {  // update time?
      M5.Lcd.fillRect(0, 0, 160, 42, BLACK);
      M5.Lcd.setTextColor(DARKGREEN);
      sprintf(buf, "%02u", dt.hour);
      M5.Lcd.drawString(buf, xpos - M5.Lcd.textWidth(buf) / 2 - 6, 3, GFXFF);// left site HH
      sprintf(buf, "%02u", dt.minute);
      M5.Lcd.drawString(buf, xpos + M5.Lcd.textWidth(buf) / 2 + 6, 3, GFXFF);// rigth site MM
      saveMinute = dt.minute;
    }
    // update ':'
    M5.Lcd.setTextColor(BLACK);
    M5.Lcd.drawString(colon, xpos - 1, 3, GFXFF);
    colon[0] = (colon[0] == ':' ? ' ' : ':') ;
    M5.Lcd.setTextColor(DARKGREEN);
    M5.Lcd.drawString(colon, xpos - 1, 3, GFXFF);
    saveSecond = dt.second;
  }
}
