/** ***************************
  * Ewald Kendziorra sr, 2019 *
  *   totally free to use     *
  *     www.kendziorra.nl     *
  *****************************/
// ESP-NOW that is direct PTOP communication without login. A perfect Mash alternative.
// Simple-ESP-NOW: an unlimited number of nodes with only 20 MAC addresses in use (or reserved).
// MAC addresses are used by Shared (0-99 on 'one' MAC address) and Paired (0-99 on a private address) nodes.
// Think it is a postal address: MAC(building) 00-19, Shared(flour) 00-99, Paired(room) 00-99, 20x100x100=200,000 nodes ;=)) 
// A node can also be used as two way relay to bridge larger distances (each node is in Access point and Station mode).
// Example adressing: "001122"; "00": MAC (00-19), "11": Shared address on MAC, "22": Private address on Shared.
//     If addressing: "0011--"; "--": the message or command go to all "0011"-Shared-Paired addresses.
// Fields seperation: '|'
// Format: "|to node address|from Me address|command/label|parammeter|...|...|"
//          ^               ^               ^             ^          ^   ^   ^
// Always use an even number of fields, an empty field you enter a space or point

char now_SSID[17] = "espnow",         // all Nodes the same ESP-NOW SSID prefix, max 10 (+6 free) chars ("espnow" or ...)
     now_Address[7] = {"000001"};     // now_Address: (MAC[1])shared[0-1], paired[2-3] and private[4-5], all fixed length with 2 numeric chars
uint8_t now_Channel = 3;              // all Nodes the same WiFi channel, 1-14 
const char now_Pswrd[] = "00000000";  // all Nodes the same password, min. 8 chars
// first the declaration, then the #include <sESPNOW.h>
#include <sESPNOW.h>   // www.kendziorra.nl

char msg[now_maxMsgLen + 1];
time_t sendTime;
uint8_t counter = 0;

void setup() {
  Serial.begin(115200);
  //Serial.setDebugOutput(true);
  while(!Serial);
  Serial.println("\n");
  if (!now_initEspnow()) {
    Serial.printf("## ESP-NOW INIT PROBLEM ##\n now_SSID:%s, now_Address:%s, now_Channel:%d \n", now_SSID, now_Address, now_Channel);
    while(1) {} // ENDLESS LOOP
  }
  Serial.printf("- ESP-NOW %s started -\n", now_SSID);
}

void loop() {
  static uint8_t n = 0;
  if (now_msgInQueue) {  // a message from WebSocket Client or other ESP-NOW Node?
    strcpy(msg, now_msgQueue[--now_msgInQueue]);  // fifo ESP-NOW message
    Serial.printf("received: %s \n", msg);
  }
  if (millis() > sendTime) {
    // addresses as "aabbcc": aa:MAC 00-19, bb:Shared 00-99, cc:Paired with Shared/Private 00-99 (or not Private but for all Shared "--")
    // TO 000000:   MAC 00, Shared 00, private 00
    // FROM now_Address, this node Private address

    if((n++ % 5) == 0) now_sendData(0, "|000000|000001|online|?|");  // is 000000 online?
    
    char to[] = "000000";  // ##### Private
    sprintf(msg, "|%s|%s|counter|%d|", to, now_Address, counter);
    char m[] = {to[0], to[0], '\0'};  // send to MAC 00 (max. 19)
    now_sendData(atoi(m), msg); 
    delay(100);
    
    strcpy(to, "0000--");  // ##### not Private but for all Shared+Paired
    sprintf(msg, "|%s|%s|to all|0000--|counter|%d|", to, now_Address, counter);
    m[0] = to[0];
    m[1] = to[1];
    now_sendData(atoi(m), msg);  // send to MAC "00-19"
    counter++;
    sendTime = millis() + 5000;
  }
  yield();
}
