﻿#ifndef _EDIT_MIDEA_H_
  #define _EDIT_MIDEA_H_

  // we use my sWiFiFull.h lib.: a very easy to use ESP32 WiFi, static IP, OTA, Time, Weather and WeSocket library
  // =====================
  #define USE_STA       // use WiFi mode STA
  #ifdef USE_STA        // only station mode
    #define USE_STAIP   // if using a static STA IP  (is needed with: USE_WS)
//    #define USE_STADNS  // use WiFi STA DNS
//  #define USE_STACHAN  // used in AP_STA mode
    #define USE_OTA     // ESP32 OTA
//  #define USE_WSER       // Web Server /** library: https://github.com/me-no-dev/ESPAsyncWebServer and https://github.com/me-no-dev/AsyncTCP **/
  #define USE_WS       // WebSockets Server         /**   // used library: https://github.com/me-no-dev/ESPAsyncWebServer and https://github.com/me-no-dev/AsyncTCP **/
//  #define USE_WSCL     // ESP32 Websocket client (sSimpleLib)
    #define USE_TIME    // NTP, date, time, timezone, DST/Summertime, Sunrise, Sunset /** my simpleLIB library: sDateTimeSun.h **/
    #define USE_WTHR    // Weather  /** my simpleLIB library: sOpenweathermap.h **/
//  #define USE_SOLARMETEO  // KNMI SOLAR weersverwachting /** sSolarMeteo.h **/
  #endif
//  #define USE_AP        // use WiFi mode AP
  #ifdef USE_AP         // only softAp mode
    #define USE_APIP     // if using a static AP IP 
    #define USE_APCHAN   // used in AP_STA mode
    #define USE_APHID    // SSID hidden
    #define USE_APMAXCON // max connections, default 4, max 10
    #define USE_NOW      // ESP-NOW only in softAP mode
  #endif
  #if defined(USE_NOW) || defined(USE_WS)
  //  #define USE_PARSER   // parse the received message to fieldnames an fieldValues 
  #endif
  #if defined(USE_STA) && defined(USE_AP) 
    #define USE_LR      // ESP32 softAP Long Range mode up to 1000m (only in AP_STA mode)
  #endif
  
  #include <sMideaIR.h>         // (simpleLib)

  const byte    irPin =    4@,  //  use a digital port
               tmpPin =   23@,  // DS18B20 in/out IO-pin
             antiYoyo =   10@;  // anti yoyo minutes
  const float tmpComp = -3.1@;  // DS18B20 temperature compensation in *C
  const char  tmpMode =  'C'@;  // DS18B20 temperature mode: C, F, K

  /** wifi, OTA and websocket **/
  const char staSsid[] = "edit_edit"@,
         staPassword[] = "edit_edit"@;
  const char myOtaID[] = "mideaDoitDev1",  // Hostname OTA
  // in ESP-NOW and WebSocket we use this message structure: "|to|from|name|val|name|val|...|...|"
  //   ### always a set of two fields ###
  const byte nFields = 8,   // the max. number of a set names/values
           nMaxChars = 60;  // the max. length of a message
  char fldIn[nFields][nMaxChars + 1],  // parse received message to fieldnames an fieldValues 
       valIn[nFields][nMaxChars + 1];  //   ... |toadr|fromadr|field-1|val-1|field-2|val-2|...|...
                                       //        fldIn  valIn   fldIn  valIn  fldIn  valIn ....
  const char *wsURI = "/MiDeA";      // WebSocket URI,       also in HTML
  const uint16_t wsPort = 82;   // Websocket Port,      also in HTML
  const IPAddress staIP(192, 168, 178, 175)@,
           staGatewayIP(192, 168, 178,   1)@,
            staSubnetIP(255, 255, 255,   0);
  /** Openweathermap **/
  char *lat = "52.0000"@,    // your location
       *lon =  "5.0000"@,
    * units = "metric",      // 'metric' or 'imperial'
       *key = "edit_edit"@;  // get a FREE key (60 calls/minute, 1,000,000 calls/month) https://openweathermap.org/price 
  /** List of timezones https://sites.google.com/a/usapiens.com/opnode/time-zones **/
  char * timeZone = "CET-1CEST-2,M3.5.0/02:00:00,M10.5.0/03:00:00"@;  // EU Amsterdam/Barcelona with Summertime (DST)
                /** "EST5EDT,M3.2.0,M11.1.0";// USA Eastern Time with DST **/
                /** "CET-1";// EU Amsterdam/Barcelona NO DST/Summertime) **/
  const time_t syncTime = 12 * 3600;  // NTP time sync interval in sec. if 0 then NOT used

  #include <sWiFiFull.h>  // simpleLib  -  always under the above declarations

  // ==========================================================
  // switching times, temperatures in *C, mode and commands
  // values can be changed and reset with the users HTML module
  // ==========================================================

  struct MIDEA {@  // init values
    byte stat =  0,  // 0: automatically, 1: continuous OFF, 2: continuous ON
         hhOn = 10,  // ON time hh
         mmOn = 15,  // ON time mm
        hhOff = 22,  // OFF time hh
        mmOff =  0,  // OFF time mm
        tmpIn = 23,  // temp inside, 17, 18, 19, 20, ...., 29, 30
       tmpOut = 25,  // temp outside
         mode =  1,  // mode       | 1:coolMod, 2:heatMod
          fan =  1,  // fan speed  | 1:lowFan,  2:midFan,  3:highFan,   4:autoFan
          cmd =  0;  // commands   | 1:onAC,    2:offAC,   3:deflectAC, 4:oscilAC, 5:turboAC, 0=nothing
  };

#endif
