// ESP32 L298N Dual H Bridge Stepper Motor Driver

#ifndef _CAR_L298N_H_
#define _CAR_L298N_H_
uint8_t carSpeedLeftPin  = 23,  carLeft1Pin = 18,  carLeft2Pin = 5,
        carSpeedRightPin = 19, carRight1Pin = 15, carRight2Pin = 2,
        carFreq = 30000, carChannelLeft = 0, carChannelRicht = 1, carResolution = 8,
        carMinCycle = 160, carMaxCycle = 250, carDelay = 20 /* usec */;
const uint8_t carStop = 0, carForward = 1, carReverse = 2, carUnchanged = 255;
uint8_t carLeftRight[2][3] {{0, 0, 0}, {0, 0, 0}};   // [0][]:left, [1][]:right, [][0]:speed-l/r ,[][0]:l/r-1 ,[][0]:l/r-2 

void carBegin(uint8_t speedLeftPin, uint8_t left1Pin, uint8_t left2Pin,
              uint8_t speedRightPin, uint8_t right1Pin, uint8_t right2Pin,
              uint8_t freq, uint8_t channelLeft, uint8_t channelRicht, uint8_t resolution,
              uint8_t  minCycle, uint8_t maxCycle, uint8_t _delay);
void carBegin();
void carSpeed();      // send the carLeftRight[0/1][0] speed values to the controller
void carSpeedTo(uint8_t leftCycle, uint8_t rightCycle);  // speed up/down from the carLeftRight[0][] and carLeftRight[1][] values
void carValues(const uint8_t cl, const uint8_t dl, const uint8_t cr, const uint8_t dr);  // set carLeftRight[2][3]: cycle l/r, direction l/r
// carValues(cl, dl, cr, dr)            :  VALUE SETTINGS
//             carSpeedLeftPin (cl LEFT): carStop, 1 to carMaxCycle, carUnchanged
//     carLeft1Pin/carLeft2Pin (dl LEFT): carStop, carForward, carReverse, carUnchanged
//           carSpeedRightPin (cr RIGHT): carStop, 1 to carMaxCycle, carUnchanged
//  carRight1Pin/carRight2Pin (dr RIGHT): carStop, carForward, carReverse, carUnchanged

void carBegin(uint8_t speedLeftPin, uint8_t left1Pin, uint8_t left2Pin,
              uint8_t speedRightPin, uint8_t right1Pin, uint8_t right2Pin,
              uint8_t freq, uint8_t channelLeft, uint8_t channelRicht, uint8_t resolution,
              uint8_t  minCycle, uint8_t maxCycle, uint8_t _delay) {
  carSpeedLeftPin = speedLeftPin;
  carLeft1Pin = left1Pin;
  carLeft2Pin = left2Pin;
  carSpeedRightPin = speedRightPin;
  carRight1Pin = right1Pin;
  carRight2Pin = right2Pin;
  carFreq = freq;
  carChannelLeft = channelLeft;
  carChannelRicht = channelRicht;
  carResolution = resolution;
  carMinCycle = minCycle;
  carMaxCycle = maxCycle;
  carDelay = _delay;
  carBegin();
}

void carBegin() {
  pinMode (carSpeedLeftPin, OUTPUT);
  pinMode (carLeft1Pin, OUTPUT);
  pinMode (carLeft2Pin, OUTPUT);
  pinMode (carSpeedRightPin, OUTPUT);
  pinMode (carRight1Pin, OUTPUT);
  pinMode (carRight2Pin, OUTPUT);
  ledcSetup(carChannelLeft, carFreq, carResolution);  
  ledcSetup(carChannelRicht, carFreq, carResolution);  
  ledcAttachPin(carSpeedLeftPin, carChannelLeft);
  ledcAttachPin(carSpeedRightPin, carChannelRicht);
}

void carSpeed() {
  ledcWrite(carChannelLeft,  carLeftRight[0][0]);
  ledcWrite(carChannelRicht, carLeftRight[1][0]);
}

void carSpeedTo(uint8_t leftCycle, uint8_t rightCycle) {
  uint8_t leftDown =  leftCycle <  carLeftRight[0][0] ? false : true,
          rightDown = rightCycle < carLeftRight[1][0] ? false : true;
  while (carLeftRight[0][0] != leftCycle || carLeftRight[1][0] != rightCycle) {
    if (leftDown)  {if (carLeftRight[0][0] < leftCycle) carLeftRight[0][0]++;}
    else if (carLeftRight[0][0] > leftCycle) carLeftRight[0][0]--;
    if (rightDown) {if (carLeftRight[1][0] < rightCycle) carLeftRight[1][0]++;}
    else if (carLeftRight[1][0] > rightCycle) carLeftRight[1][0]--;
    carSpeed();
    delay(carDelay);
  }
}

void carValues(const uint8_t cl, const uint8_t dl, const uint8_t cr, const uint8_t dr) {
  switch (dl) {
    case carStop:      carLeftRight[0][1] = 0; carLeftRight[0][2] = 0; break;
    case carForward:   carLeftRight[0][1] = 1; carLeftRight[0][2] = 0; break;
    case carReverse:   carLeftRight[0][1] = 0; carLeftRight[0][2] = 1; break;
    case carUnchanged: break;
    default: break;
  } 
  switch (dr) {
    case carStop:      carLeftRight[1][1] = 0; carLeftRight[1][2] = 0; break;
    case carForward:   carLeftRight[1][1] = 1; carLeftRight[1][2] = 0; break;
    case carReverse:   carLeftRight[1][1] = 0; carLeftRight[1][2] = 1; break;
    case carUnchanged: break;
    default: break;
  } 
  if (cl != carUnchanged) carLeftRight[0][0] = cl;  // carStop, 1 to carMaxCycle, carUnchanged
  if (cr != carUnchanged) carLeftRight[1][0] = cl;  // carStop, 1 to carMaxCycle, carUnchanged
  if (dl != carUnchanged) {
    digitalWrite(carLeft1Pin,  carLeftRight[0][1]);
    digitalWrite(carRight1Pin, carLeftRight[1][1]);
  }
  if (dr != carUnchanged) {
    digitalWrite(carLeft2Pin,  carLeftRight[0][2]);
    digitalWrite(carRight2Pin, carLeftRight[1][2]);
  }
}

#endif
