////////////////// sDS18B20.h ///////////////////
//        Ewald sr, www.kendziorra.nl          //
//   a very simple DS18B20 mini "1-Wire" lib.  //
//  Nov 2023, completely free to edit and use  //
//---------------------------------------------//
//    use it as class for a single DS18B20     //
//              -----      -------             //
//        each sensor it's own IO-port         //
//         normal mode (NOT parasite)          //
// No other 1-wire or Dallas library is needed //
//---------------------------------------------//
//      tested and OK with an ESP32 Dev        //
////////////////// sDS18B20.h ///////////////////

#ifndef _S_DS18B20_H_
  #define _S_DS18B20_H_

  uint8_t tPin = 19;
  float tComp;
  
  float getTemp(uint8_t pin, float comp = 0.0);
  float getTemp(); 
  bool DS18B20start(void);
  void DS18B20write(uint8_t b);
  uint8_t DS18B20read (void);
  
  float getTemp(uint8_t pin, float comp) {  // default tPin and tComp
    tPin = pin;
    tComp = comp;
    return getTemp();
  }
  
  float getTemp() {  // default tPin and comp
    noInterrupts();
    if (DS18B20start()) {;
      delay(1);
      DS18B20write(0xCC);  // skip ROM
      DS18B20write(0x44);  // convert t
      delay(800);
      if (DS18B20start()) {
        delay(1);
        DS18B20write(0xCC);  // skip ROM
        DS18B20write(0xBE);  // Read Scratch-pad
        uint8_t val[2] = {DS18B20read(), DS18B20read()};
        interrupts();
        return (float)((val[1] << 8) | val[0]) / 16.0 + tComp;
      }
    }
    interrupts();
    return 999.9;
  }
    
  bool DS18B20start (void) {
    bool f = false;
    pinMode(tPin, OUTPUT);
    digitalWrite(tPin, LOW);
    delayMicroseconds(480);
    pinMode(tPin, INPUT);
    delayMicroseconds(80);
    if (!(digitalRead(tPin))) f = true;
    delayMicroseconds(400);
    return f;
  }
  
  void DS18B20write (uint8_t b) {
    pinMode(tPin, OUTPUT);
    for (uint8_t i = 0; i < 8; i++) {
      pinMode(tPin, OUTPUT);
      digitalWrite(tPin, LOW);
      if ((b & (1 << i)) != 0) { // if the bit is high
        delayMicroseconds(1);
        pinMode(tPin, INPUT);
        delayMicroseconds(50);
      }
      else { // if the bit is low
        delayMicroseconds(50);
        pinMode(tPin, INPUT);
      }
    }
  }
  
  uint8_t DS18B20read (void) {
    uint8_t val = 0;
    pinMode(tPin, INPUT);
    for (int i = 0; i < 8; i++) {
      pinMode(tPin, OUTPUT);
      digitalWrite(tPin, LOW);
      delayMicroseconds(1);
      pinMode(tPin, INPUT);
      if (digitalRead(tPin)) val |= 1 << i;
      delayMicroseconds(50);
    }
    return val;
  }  

#endif
