/** simpleDateTimeSun library - www.kendziorra.nl free to use May 2021 *
  *                 ESP32 epoch time in milliseconds                   **/

#ifndef _S_MSEC_TIME_
#define _S_MSEC_TIME_

#include <WiFi.h>
#include <sys/time.h>

/***************************************************
** you can change these default values in setup() **
****************************************************/
// ***** strftime: http://www.cplusplus.com/reference/ctime/strftime
// ***** we assume by default that Monday is the first day of the week (ISO 8601)
char _week = 'V';  // strftime week number (ISO 8601)
char _wday = 'u';  // strftime day of the week (ISO 8601)
char _monthsLstr[12][21] = {"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"};
char _monthsSstr[12][11] = {"Jan", "Feb", "Mar", "Apr", "May", "June", "July", "Aug", "Sept", "Oct", "Nov", "Dec"};
char _daysLstr[7][21] = {"Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"};  // strftime _wday: 'u' (ISO 8601)
char _daysSstr[7][11] = {"Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"};  // strftime _wday: 'u' (ISO 8601)
/*****************************************
 **  END change these values in setup() **
 *****************************************/
const char * _NTPservers[21] = {"time.google.com", "pool.ntp.org", "time.nist.gov"};  // standard NTP Servers
enum {
  sOneminute = 60,
  sOnehour = sOneminute * 60,
  sOneday = sOnehour * 24
};
struct sEpoch {      // timestamp and milliseconds
  time_t sec;        // UNIX timestamp
  int16_t msec;      // milliseconds 000-999
};
struct sRiseSet {    // output: SunRise and SunSet local or UTC
  time_t rise,       // SunRise local or UTC
         set;        // SunSet  local or UTC
};
struct sEquiSol {    // output: Equinox and Solstice (as UTC/GMT time)
  sEpoch spring,     // Equinox,  March
         summer,     // Solstice, June
         autumn,     // Equinox,  September
         winter;     // Solstice, December
};
struct sSolPosOut {  // output: True North location of the Sun in the sky
  double azimuth,    // the angle of the sun around the horizon, usually measured from true north and increasing eastward
         altitude;   // the angle between the sun and the observer's local horizon
};
struct sSolPosIn {   // input: True North location of the Sun in the sky
  time_t time;       // local time
  double temp,       // you can get it with my sOpenweathermap.h lib.(free key on "http://api.openweathermap.org/data/2.5/weather?")
         pres,       // you can get it with my sOpenweathermap.h lib.
         zone,       // seconds: use "sDateTime.offset" or a "local-UTC" time
         lat,        // location North/South parallel to the Equator 
         lon;        // location East/West of the 0°Prime Meridian (The meridian of the British Royal Observatory in Greenwich)
};
struct sDateTime {  // local/UTC time and date
  // set by function: sDateTime sTimeDate(bool _gmt, sEpoch _ep)
  time_t sec;  // UNIX timestamp (it's a int32_t)
               // GMT < 0, MIN: 13 December 1901 20:45:52
               // GMT == 0,   :  1 January  1970 00:00:00
               // GMT > 0, MAX: 19 January  2038 03:14:07
  int8_t second,    // seconds 00-59
         minute,    // minutes 00-59
         hour,      // hours 00-23
         day,       // day 01-31
         month,     // month 01-12
         week,      // week 01-53
         wday;      // weekday 1-7
  int16_t msec,     // milliseconds 000-999
          year,     // year 1901-2038
          yday;     // yearday 001-366
  // timezone and DST
  time_t zone,      // seconds
         dst,       // seconds
         offset;    // zone + dst (sec - offset = UTC/GMT)
  // Long/Short month/day names
  char monthL[21],  // month Long name
       monthS[11],  // month Short name
       dayL[21],    // day Long name
       dayS[11];    // day Short name
};
struct sSMHD {  // split timestamp to time and days
  // set by function: sSMHD sTimeSplit(time_t _t)
  int8_t seconds,   // seconds 00-59
         minutes,   // seconds 00-59
         hours;     // hours 00-23
  int16_t days;     // days 0-n
};
//////////////////////////// private /////////////////////////////
sEpoch sEp = {.sec = 0, .msec = -1};                            //
// strftime: http://www.cplusplus.com/reference/ctime/strftime/ //
// for numerical values only                                    //
const char _hour   = 'H',  // strftime hour                     //
           _minute = 'M',  // strftime minute                   //
           _second = 'S',  // strftime second                   //
           _day    = 'd',  // strftime day of the month         //
           _month  = 'm',  // strftime month                    //
           _year   = 'Y',  // strftime year                     //
           _yday   = 'j';  // strftime day of the year          //
//////////////////////////////////////////////////////////////////
/*************************************
 ** change this defaults in setup() **
 *************************************/
void sSetWeek(const char _wk = _week);             // change strftime week code
void sSetWday(const char _wd = _wday);             // change strftime weekday code
void sSetMonthDayStr(const char _mL[12][21], const char _mS[12][11], const char _dL[7][21], const char _dS[7][11]);  // change long and short month and day names
/*******************************************
 **  END change this defaults  in setup() **
 *******************************************/
void sSetTz(char* tz);                     // set Time Zone
bool sGetNTP(char * _timeZone);            // get time from server
void sSetEpoch(sEpoch _ep);                // set time manual, use sMakeTime()
uint64_t sTimestampInt(sEpoch _ep = sEp);  // timestamp in milliseconds
String sTimestampStr(sEpoch _ep = sEp);    // timestampString in milliseconds
sEpoch sMakeTime(int64_t _msec);           // time from a timestamp in milliseconds
sEpoch sMakeTime(int8_t h, int8_t m, int8_t s, int16_t ms, int8_t D, int8_t M, int16_t Y);  // calc a time
sEpoch sNow(sEpoch _ep = sEp);             // current time or current time +/- t-offset in seconds
sEpoch sAbsDiff(sEpoch _epX, sEpoch _epY = sEp);  // absolute difference between _X and _Y
sSMHD sTimeSplit(time_t t);                // split a seconds time in days, hours, minutes and seconds
//////////////// private /////////////////////////////////
int16_t sGetTval(time_t t,  char cc, bool gmt = false); //
//////////////////////////////////////////////////////////
sDateTime sTimeDate(bool _gmt, sEpoch _ep = sEp);  // get a set of local/gmt time and date values
time_t now(time_t t = 0);  // as ep.sec from sEpoch ep = sNow();
/** get GMT + zone + dst **/
uint16_t year(time_t t = now());
uint8_t month(time_t t = now());
uint8_t day(time_t t = now());
uint8_t hour(time_t t = now());
uint8_t minute(time_t t = now());
uint8_t second(time_t t = now());
uint8_t weekday(time_t t = now());
uint8_t yweek(time_t t = now());
uint16_t yday(time_t t = now());
/** get GMT **/
uint16_t yearGmt(time_t t = now());
uint8_t monthGmt(time_t t = now());
uint8_t dayGmt(time_t t = now());
uint8_t hourGmt(time_t t = now());
uint8_t minuteGmt(time_t t = now());
uint8_t secondGmt(time_t t = now());
uint8_t weekdayGmt(time_t t = now());
uint8_t yweekGmt(time_t t = now());
uint16_t ydayGmt(time_t t = now());
/*************/
sRiseSet sSun(time_t t, double lat, double lon, bool gmt = false);  // calc rise and set times
bool sDaylight(time_t t, sRiseSet rs);  // is it sun down time 
time_t sCalcSun(double lat, double lon, uint16_t N, bool rise);    // calc sun time
sEquiSol sCalcEquiSol(uint16_t Y);  // Equinox and Solstice calculation: spring, summer, autumn, winter  (as UTC/GMT time)
sSolPosOut sSolPos(sSolPosIn _sp);  // Calculates the True North location of the Sun in the sky for any given location, day, and time. Valid from the year 1950 to 2050
/*************************************
 ** change this defaults in setup() **
 *************************************/
void sSetWeek(const char _wk) {  // change strftime week code
  _week = _wk;
}

void sSetWday(const char _wd) {  // change strftime weekday code
  _wday = _wd;
}

void sSetMonthDayStr(const char _mL[12][21], const char _mS[12][11], const char _dL[7][21], const char _dS[7][11]) {  // change long and short month and day names
  memcpy (_monthsLstr, _mL, (12 * 21));
  memcpy (_monthsSstr, _mS, (12 * 11));
  memcpy (_daysLstr, _dL, (7 * 21));
  memcpy (_daysSstr, _dS, (7 * 11));
}
/*******************************************
 **  END change this defaults  in setup() **
 *******************************************/

void sSetTz(char * tz) {  // set Time Zone
  unsetenv("TZ");
  setenv("TZ", tz, 1);
  tzset();
}

bool sGetNTP(char * _timeZone) {  // get time from server, auto update: use interval test in loop() and call sGetNTP(timeZone)
  if (!WiFi.isConnected()) {
    WiFi.begin();
    delay(500);
  }
  if (WiFi.isConnected()) {
    configTzTime(_timeZone, (char*)_NTPservers[0], (char*)_NTPservers[1], (char*)_NTPservers[2]);
    timeval ts;
    gettimeofday(&ts, nullptr);
    if (ts.tv_sec < 1577836800) return false;  // < Jan 1 2020 00:00:00
    return true;
  }
  return false;
}

void sSetEpoch(sEpoch _ep) {  // set time manual, use sMakeTime()
  timeval ts = {ts.tv_sec = _ep.sec};
  settimeofday(&ts, nullptr);
}

uint64_t sTimestampInt(sEpoch _ep) {  // timestamp in milliseconds
  if (_ep.msec == -1) _ep = sNow();
  return (uint64_t)_ep.sec * 1000 + _ep.msec;
}

String sTimestampStr(sEpoch _ep) {  // timestampString in milliseconds
  if (_ep.msec == -1) _ep = sNow();
  char c[15];
  sprintf(c, "%d%03d", _ep.sec, _ep.msec);
  String s = String(c);
  return s;
}

sEpoch sMakeTime(int64_t _msec) {  // time from a timestamp in milliseconds
  sEpoch d;
  d.sec = _msec / 1000;
  d.msec = _msec % 1000;
  return d;
}

sEpoch sMakeTime(int8_t h, int8_t m, int8_t s, int16_t ms, int8_t D, int8_t M, int16_t Y) {  // calc a time
  sEpoch _ep;
  tm tmp;
  for (uint8_t i = 0; i < 2; i++) {  // avoid time zone problem
    tmp.tm_sec = s;
    tmp.tm_min = m;
    tmp.tm_hour = h;
    tmp.tm_mday = D;
    tmp.tm_mon = M - 1;
    tmp.tm_year = Y - 1900;
    time_t t = mktime(&tmp);  // avoid time zone problem
  }
  _ep.sec = mktime(&tmp) + ms / 1000;
  _ep.msec = ms % 1000;
  return _ep;
}

sEpoch sNow(sEpoch _ep) {  // current time or current time +/- offset in msec
  timeval ts;
  gettimeofday(&ts, nullptr);
  if (_ep.msec == -1) {
    _ep.sec = ts.tv_sec;
    _ep.msec = (int16_t)(ts.tv_usec / 1000);
  }
  else {
    _ep.sec += ts.tv_sec;
    _ep.msec += (int16_t)(ts.tv_usec / 1000);
  }
  return _ep;
}

sEpoch sAbsDiff(sEpoch _epX, sEpoch _epY) {  // absolute difference between _X and _Y
  if (_epY.msec == -1) _epY = sNow();
  return sMakeTime(abs(((int64_t)_epX.sec * 1000 + _epX.msec) -
                       ((int64_t)_epY.sec * 1000 + _epY.msec)));
}

sSMHD sTimeSplit(time_t _t) {  // split a seconds time in days, hours, minutes and seconds 
  sSMHD n;
  _t = abs(_t);
  n.days = _t / sOneday;
  _t %= sOneday;
  n.hours = _t / sOnehour;
  _t %= sOnehour;
  n.minutes = _t / sOneminute;
  _t %= sOneminute;
  n.seconds = _t;
  return n;
}

/////////////////// private ///////////////////////
int16_t sGetTval(time_t t, char cc, bool gmt) {  //
  char b[8], c[3] = "%";                         //
  tm * tmp = (gmt ? gmtime(&t) : localtime(&t)); //
  c[1] = cc;                                     //
  strftime(b, 5, c, tmp);                        //
  return atoi(b);                                //
}                                                //
///////////////////////////////////////////////////
sDateTime sTimeDate(bool getGmt, sEpoch _ep) {  // get a set of local/gmt time and date values
  char * _tz,
          tzGMT[] = "GMT+0";
  if (getGmt) {
    _tz = getenv("TZ");  // if UTC/GMT: save timezone and DST/Summertime
    sSetTz(tzGMT);       // timezone = 0 sec, DST/Summertime = 0 sec
  }
  if (_ep.msec == -1) _ep = sNow();
  sDateTime dt;
  dt.sec    = _ep.sec;
  dt.msec   = _ep.msec;
  dt.second = sGetTval(_ep.sec, _second);
  dt.minute = sGetTval(_ep.sec, _minute);
  dt.hour   = sGetTval(_ep.sec, _hour);
  dt.day    = sGetTval(_ep.sec, _day);
  dt.month  = sGetTval(_ep.sec, _month);
  dt.year   = sGetTval(_ep.sec, _year);
  dt.week   = sGetTval(_ep.sec, _week);
  dt.wday   = sGetTval(_ep.sec, _wday);
  dt.yday   = sGetTval(_ep.sec, _yday);
  if (!getGmt) {
    time_t gmt;
    tm *ptm;
    ptm = gmtime(&_ep.sec);
    gmt = mktime(ptm);
    dt.zone = (time_t)difftime(_ep.sec, gmt);
    ptm = gmtime(&_ep.sec);
    ptm->tm_isdst = -1;
    gmt = mktime(ptm);
    dt.offset = difftime(_ep.sec, gmt);
    dt.dst = abs(dt.offset - dt.zone);
  }
  else {
    dt.zone = 0;
    dt.offset = 0;
    dt.dst = 0;
    sSetTz(_tz);  // if UTC/GMT: restore timezone and DST/Summertime
  }
  strcpy(dt.monthL, _monthsLstr[dt.month - 1]);
  strcpy(dt.monthS, _monthsSstr[dt.month - 1]);
  strcpy(dt.dayL, _daysLstr[dt.wday - 1]);
  strcpy(dt.dayS, _daysSstr[dt.wday - 1]);
  return dt;
}
//////// as if using Arduino "time.h" or "TimeLib.h"
time_t now(time_t t) {  // t: + or - offset to now()
  time_t t1;
  sEpoch _ep = sNow();
  t1 = _ep.sec;
  return t1 + t;
}

/** get GMT + zone + dst **/
uint16_t year(time_t t) {
  return sGetTval(t, _year);
}

uint8_t month(time_t t) {
  return sGetTval(t, _month);  
}

uint8_t day(time_t t) {
  return sGetTval(t, _day);   
}

uint8_t hour(time_t t) {
  return sGetTval(t, _hour);   
}

uint8_t minute(time_t t) {
  return sGetTval(t, _minute);   
}

uint8_t second(time_t t) {
  return sGetTval(t, _second);   
}

uint8_t weekday(time_t t) {
  return sGetTval(t, _wday);   
}

//////// extra 
uint8_t yweek(time_t t) {
  return sGetTval(t, _week);   
}

uint16_t yday(time_t t) {
  return sGetTval(t, _yday);   
}

/** get GMT **/
uint16_t yearGmt(time_t t) {
  return sGetTval(t, _year, true);
}

uint8_t monthGmt(time_t t) {
  return sGetTval(t, _month, true);  
}

uint8_t dayGmt(time_t t) {
  return sGetTval(t, _day, true);   
}

uint8_t hourGmt(time_t t) {
  return sGetTval(t, _hour, true);   
}

uint8_t minuteGmt(time_t t) {
  return sGetTval(t, _minute, true);   
}

uint8_t secondGmt(time_t t) {
  return sGetTval(t, _second, true);   
}

uint8_t weekdayGmt(time_t t) {
  return sGetTval(t, _wday, true);   
}

//////// extra 
uint8_t yweekGmt(time_t t) {
  return sGetTval(t, _week, true);   
}

uint16_t ydayGmt(time_t t) {
  return sGetTval(t, _yday, true);   
}
/*************/

/** calc sunrise and sunset times **/
sRiseSet sSun(time_t t, double lat, double lon, bool gmt) {  // calc rise and set times
  sRiseSet sun;
  sEpoch ep = {.sec = t, .msec = 0};
  sDateTime dt = sTimeDate(gmt, ep);
  sun.rise = sCalcSun(lat, lon, yday(t), true)  + dt.offset;  // get SunRise time 
  sun.set  = sCalcSun(lat, lon, yday(t), false) + dt.offset;  // get SunSet time
  return sun;  // result: _sun.rise and _sun.set
}

bool sDaylight(time_t t, sRiseSet rs) {  // is it sun down time 
  time_t m[3] = {hour(t) * 60 + minute(t), 
                   hour(rs.rise) * 60 + minute(rs.rise),
                   hour(rs.set) * 60 + minute(rs.set)};
  return m[0] > m[1] && m[0] < m[2];
}

/** Source:
 *  Almanac for Computers, 1990
 *  published by Nautical Almanac Office
 *  United States Naval Observatory
 *  Washington, DC 20392
 *  Adapted by Ewald Kendziorra sr, www.kendziorra.nl
 **/
time_t sCalcSun(double lat, double lon, uint16_t N, bool rise) {
  const double _zenith = 90.883,
               _toRad = 0.01745329,
               _toDeg = 57.2957786;
  double lngHour = lon / 15.0;
  double t =  N + ((rise ? 6.0 : 18.0) - lngHour) / 24.0;
  double M = (0.9856 * t) - 3.289;
  double L = M + (1.916 * sin(M * _toRad)) + (0.020 * sin(2 * M * _toRad)) + 282.634;
  L += (L < 0 ? 360 : L > 360 ? -360 : 0); 
  double RA = atan(0.91764 * tan(L * _toRad)) * _toDeg;
  RA += (RA < 0 ? 360 : RA > 360 ? -360 : 0);
  double Lquadrant = (floor(L / 90.0)) * 90.0;
  double RAquadrant = (floor(RA / 90.0)) * 90.0;
  RA = (RA + Lquadrant - RAquadrant) / 15.0;
  double sinDec = 0.39782 * sin(L * _toRad);
  double cosDec = cos(asin(sinDec));
  double cosH = (cos(_zenith * _toRad) - (sinDec * sin(lat * _toRad))) / (cosDec * cos(lat * _toRad));
  double H = (rise ? (360.0 - acos(cosH) * _toDeg) : (acos(cosH) * _toDeg)) / 15; 
  double T = H + RA - (0.06571 * t) - 6.622;
  double UT = T - lngHour;
  UT +=  (UT < 0 ? 24 : UT > 24 ? -24 : 0);
  //return UT;  // UTC decimal time 
  UT -= 1.0;
  uint8_t hh = (uint8_t)UT;
  uint8_t mm = (uint8_t)((UT - hh) * 60);
  return hh * sOnehour + mm * sOneminute;
}

/** calc spring, summer, autumn, winter (as UTC/GMT time) **/
/** Equinox and Solstice calculation (UTC/GMT time)
 *  Based on: Astronomical Algorithms, by Jean Meeus, 2nd ed, 1998
 *  Adapted by Ewald Kendziorra sr, www.kendziorra.nl
 **/
sEquiSol sCalcEquiSol(uint16_t Y) {
  const double y = (Y - 2000.0) / 1000.0,
            J[4] = {2451623.80984 + (365242.37404 + (0.05169  - (0.00411 - 0.00057*y)*y)*y)*y,
                    2451716.56767 + (365241.62603 + (0.00325  - (0.00888 - 0.00030*y)*y)*y)*y,
                    2451810.21715 + (365242.01767 + (0.11575  - (0.00337 - 0.00078*y)*y)*y)*y,
                    2451900.05952 + (365242.74049 + (0.06223  - (0.00823 - 0.00032*y)*y)*y)*y},
            S[4] = {216.41823823654445, 600.260100916502, 749.3030751167832, 1031.0192170678533};
  sEquiSol es;
  for (uint8_t n = 0; n < 4; n++) {
    double T, W, L, R;
    float h, i, j;
    uint32_t A, C, D, E;
    uint8_t B, F;
    T = (J[n] - 2451545.0) / 36525.0,
    W = DEG_TO_RAD * (35999.373 * T - 2.47),
    L = 1.0 + 0.0334 * cos(W) + 0.0007 * cos(2.0 * W * DEG_TO_RAD),
    R = J[n] + (0.00001 * S[n]) / L - (66.0 + (Y - 2000.0) * 1.0) / 86400.0,
    A = floor(R + 0.5),
    B = floor((A - 1867216.25) / 36524.25),
    C = A + B - floor(B / 4.0) + 1525.0,
    D = floor((C - 122.1) / 365.25),
    E = 365.0 * D + floor(D / 4.0),
    F = floor((C - E) / 30.6001),
    h = 24.0 * (R + 0.5 - A),
    i = abs(h) - floor(abs(h)),
    j = F - 1 - 12 * floor(F / 14);
    sEpoch dd = sMakeTime(floor(h), floor(round(i * 60.0)), 0, 0, floor(C - E + 0.5) - floor(30.6001 * F), j, D - 4715 - floor((7 + j) / 10));  // calc time
    switch (n) {
      case 0: es.spring.sec = dd.sec; break;
      case 1: es.summer.sec = dd.sec; break;
      case 2: es.autumn.sec = dd.sec; break;
      case 3: es.winter.sec = dd.sec; break;
    }
  }
  return es;
}

/************ position of the Sun (Solar tracker) ************/
/*  Based on SOLPOS NREL C code 
 *  Calculates the location of the Sun in the sky for any given 
 *     location, day, and time. Valid from the year 1950 to 2050, with an 
 *     uncertainty of +/- 0.01 degrees.    
 *  Orginal code from:
 *  Martin Rymes
 *  National Renewable Energy Laboratory
 *  25 March 1998
 *  Please read the disclaimer www.nrel.gov/disclaimer.html before using this product.
 *  The RReDC has more information about SOLPOS and the C source code.
 *  http://rredc.nrel.gov/solar/codesandalgorithms/solpos/
 *  https://mid_cdmz.nrel.gov/solpos/
 *  Adapted by Ewald Kendziorra sr, www.kendziorra.nl
 **/
sSolPosOut sSolPos(sSolPosIn _sp) {
  sSolPosOut _su;
  double _utime = hour(_sp.time) + minute(_sp.time) * 100 / 60.0/100 + second(_sp.time)/3600 - _sp.zone / 3600;
  double _delta = year(_sp.time) - 1949;
  double _julday = 32916.5 + _delta * 365.0 + (int)(_delta / 4.0) + yday(_sp.time) + _utime / 24.0;
  double _ectime = _julday - 51545.0;
  double _mnlong = 280.460 + 0.9856474 * _ectime - 360.0 *(int)(_mnlong / 360.0);
  if(_mnlong < 0.0) _mnlong += 360.0;
  double _mnanom = 357.528 + 0.9856003 * _ectime - 360.0 *(int)(_mnanom / 360.0);
  if(_mnanom < 0.0) _mnanom += 360.0;
  double _eclong = _mnlong + 1.915 * sin(_mnanom * DEG_TO_RAD) + 0.020 * sin(2.0 * _mnanom * DEG_TO_RAD)- 360.0 *(int)(_eclong / 360.0);
  if(_eclong < 0.0) _eclong += 360.0;
  double _ecobli = 23.439 - 4.0e-07 * _ectime;
  double _declin = RAD_TO_DEG * asin(sin(_ecobli * DEG_TO_RAD) * sin(_eclong * DEG_TO_RAD));
  double _rascen =  RAD_TO_DEG * atan2(cos(DEG_TO_RAD * _ecobli) * sin(DEG_TO_RAD * _eclong), cos(DEG_TO_RAD * _eclong));
  if(_rascen < 0.0) _rascen += 360.0;
  double _gmst = 6.697375 + 0.0657098242 * _ectime + _utime - 24.0 *(int)(_gmst / 24.0);
  if(_gmst < 0.0) _gmst += 24.0;
  double _lmst = _gmst * 15.0 + _sp.lon - 360.0 *(int)(_lmst / 360.0);
  if(_lmst < 0.) _lmst += 360.0;
  double _hrang = _lmst - _rascen;
  if(_hrang < -180.0) _hrang += 360.0;
  else if(_hrang > 180.0) _hrang -= 360.0;
  double _cl = cos(DEG_TO_RAD * _sp.lat);
  double _sl = sin(DEG_TO_RAD * _sp.lat);
  double _sd = sin(DEG_TO_RAD * _declin);
  double _cz = _sd * _sl + cos(DEG_TO_RAD * _declin) * _cl * cos(DEG_TO_RAD * _hrang);
  if(fabs(_cz) > 1.0) {
    if(_cz >= 0.0) _cz =  1.0;
    else _cz = -1.0;
  }
  double _zenetr = acos(_cz) * RAD_TO_DEG;
  if(_zenetr > 99.0) _zenetr = 99.0;
  double _elevetr = 90.0 - _zenetr;
  double _azim = 180.0;
  double _cecl = cos(DEG_TO_RAD * _elevetr) * _cl;
  if(fabs(_cecl) >= 0.001) {
    double _ca =(sin(DEG_TO_RAD * _elevetr) * _sl - _sd) / _cecl;
    if(_ca > 1.0) _ca = 1.0;
    else if(_ca < -1.0) _ca = -1.0;
    _azim = 180.0 - acos(_ca) * RAD_TO_DEG; 
    if(_hrang > 0) _azim = 360.0 - _azim;
  }
  double _refcor;
  if(_elevetr > 85.0) _refcor = 0.0;
  else {
    double _tanelev = tan(DEG_TO_RAD * _elevetr);
    if(_elevetr >= 5.0) _refcor = 58.1 / _tanelev - 0.07 /(pow(_tanelev, 3)) + 0.000086 /(pow(_tanelev, 5));
    else if(_elevetr >= -0.575) _refcor = 1735.0 + _elevetr *(-518.2 + _elevetr *(103.4 + _elevetr *(-12.79 + _elevetr * 0.711)));
    else _refcor = -20.774 / _tanelev;
    double _prestemp =(_sp.pres * 283.0) /(1013.0 *(273.0 + _sp.temp));
    _refcor *= _prestemp / 3600.0;
  }
  double _elevref = _elevetr + _refcor;
  if(_elevref < -9.0) _elevref = -9.0;
  _su.azimuth = (_azim >= -1 ? _azim : -2);     // Solar azimuth angle: N=0, E=90, S=180,W=270
  _su.altitude = (_azim >= -1 ? _elevref: -2);  // Solar elevation angle, degrees from horizon, refracted
  return _su;
}
  
#endif