/** ***************************
  * Ewald Kendziorra sr, 2019 *
  *   totally free to use     *
  *     www.kendziorra.nl     *
  *****************************/

#ifdef ESP32
  #ifndef _S_ESP_NOW_H_
    #define _S_ESP_NOW_H_

    #include <WiFi.h>
    #include <esp_wifi.h>
    #include <esp_now.h>

    uint8_t now_macID,                            // the node ID and 2nd MAC address number
            now_msgInQueue = 0,                   // number off messages in queue
            now_nodeMAC[]  = {0, 0, 2, 3, 4, 5};  // all slaves the same MAX except position [0] and [1] (changed by code)
    const uint8_t now_maxSlaves   = 20,           // the max. number of slaves
                  now_maxMsgLen   = 249,          // the max. number of message chars
                  now_msgQueueLen = 10;           // max. number off messages in queue
    char now_PairedAddr[7],
         now_msgQueue[now_msgQueueLen][now_maxMsgLen + 1];  // holt the received message

    enum {
      now_Paired,
      now_Private
    };

    esp_now_peer_info_t slave;
    const esp_now_peer_info_t *peer = &slave;

    bool now_initEspnow();
    bool now_espnowOn();
    void now_espnowOff();
    void now_sendData(uint8_t _slave, const char *_msg);
    void now_onDataRecv(const uint8_t *mac, const uint8_t *data, int len);

    bool now_initEspnow() {
      uint8_t l = strlen(now_SSID);
      if (l == 0 || l > 10) return false;
      l = strlen(now_Address);
      if (l != 6) return false;
      for (uint8_t i = 0; i < 6; i++) {
        if (now_Address[i] < '0' || now_Address[i] > '9') return false;
      }
      char c[3] = {now_Address[0], now_Address[1]};
      now_macID = atoi(c);
      strcpy(now_PairedAddr, now_Address);
      now_PairedAddr[4] = '-';
      now_PairedAddr[5] = '-';
      if (now_Channel < 1 || now_Channel > 13) return false;
      return now_espnowOn();
    }

    bool now_espnowOn() {
      bool ok = true;
      WiFi.persistent(false);
      WiFi.disconnect();
      WiFi.mode(WIFI_OFF);
      WiFi.mode(WIFI_AP_STA);
      now_nodeMAC[0] = 0;
      now_nodeMAC[1] = now_macID;  // identification #
      esp_wifi_set_mac(ESP_IF_WIFI_AP, now_nodeMAC);
      WiFi.softAP(now_SSID, now_Pswrd, now_Channel, 1);
      Serial.println(" APmac: " + String(WiFi.softAPmacAddress()));
      Serial.println("STAmac: " + String(WiFi.macAddress()));
      if(esp_now_init() == ESP_OK) Serial.println("ESPNow Init Success!");
      else {
        Serial.println("ESPNow Init Failed....");
        ok = false;
      }
      Serial.print("Add Node: ");
      for (uint8_t i = 0; i < now_maxSlaves; i++) {
        now_nodeMAC[1] = i;
        memcpy(&slave.peer_addr, &now_nodeMAC, 6);
        slave.channel = now_Channel;
        slave.encrypt = 0;
        if( esp_now_add_peer(peer) == ESP_OK) Serial.printf("%u%s", i, (i < (now_maxSlaves - 1) ? ", " : "\n"));
      }
      esp_now_register_recv_cb(now_onDataRecv);
      return ok;
    }

    void now_espnowOff() {
      esp_now_deinit();
    }

    void now_sendData(uint8_t _mac, const char *_msg) {
      now_nodeMAC[0] = 0;
      now_nodeMAC[1] = _mac;
      if (esp_now_send(now_nodeMAC, (uint8_t*)_msg, strlen(_msg)) != ESP_OK) Serial.printf("\nDID NOT SEND %s\n", _msg);
    }

    void now_onDataRecv(const uint8_t *mac, const uint8_t *data, int len) {
      bool toBuf = false;
      char _msg[len + 2];
      snprintf(_msg, len + 1, "%s", (char*)data);
      /** test format and address **/
      if (_msg[0] == '|' && _msg[strlen(_msg) - 1] == '|' && _msg[7] == '|') {
        char c[7] = {_msg[1], _msg[2], _msg[3],_msg[4], _msg[5], _msg[6]};  // address test on: _MAC, _ID1 and _ID2
        if ((strcmp(c, now_Address) == 0) || (strcmp(c, now_PairedAddr) == 0)) toBuf = true;
      }
      else if (strcmp(now_Address, "000000") == 0) {  // Node "000000" then test on "|WS|"
        char c[5] = {_msg[0], _msg[1], _msg[2], _msg[3]};
        if(strcmp(c, "|WS|") == 0) toBuf = true;
      }
      if (toBuf) {
        if (now_msgInQueue == now_msgQueueLen) {  // queue full?
          for (uint8_t i = 1; i < now_msgQueueLen; i++) strcpy(now_msgQueue[i - 1], now_msgQueue[i]); // delete oldest msg
          now_msgInQueue--;
        }
        strcpy(now_msgQueue[now_msgInQueue++], _msg);
      }
    }

  #endif
#endif
