/** ***************************
  * Ewald Kendziorra sr, 2019 *
  *   totally free to use     *
  *     www.kendziorra.nl     *
  *****************************/

#ifdef ESP32
  #ifndef _S_ESP_NOW_H_
    #define _S_ESP_NOW_H_
    
    #include <WiFi.h>
    #include <esp_wifi.h>
    #include <esp_now.h>
              
    uint8_t now_macID,
            now_nodeMAC[]  = {0, 0, 2, 3, 4, 5};
    const uint8_t now_maxSlaves = 20;  // min:1, max:20
    esp_now_peer_info_t slave;
    const esp_now_peer_info_t *peer = &slave;

    bool now_initEspnow();
    void now_sendData(uint8_t _mac, const char *_msg);
    void now_onDataRecv(const uint8_t *mac, const uint8_t *data, int len);
    void now_onDataSent(const uint8_t *mac_addr, esp_now_send_status_t status);

    bool now_initEspnow() {
      bool ok = true;
      char c[3] = {now_Address[0], now_Address[1]};
      now_macID = atoi(c);
      WiFi.persistent(false);
      WiFi.disconnect();
      WiFi.mode(WIFI_OFF);
      WiFi.mode(WIFI_AP_STA);
      now_nodeMAC[0] = 0;
      now_nodeMAC[1] = now_macID;  // identification #
      esp_wifi_set_mac(ESP_IF_WIFI_AP, now_nodeMAC);
      WiFi.softAP(now_SSID, now_Pswrd, now_Channel, 1);
      Serial.println(" APmac: " + String(WiFi.softAPmacAddress()));
      Serial.println("STAmac: " + String(WiFi.macAddress()));
      if(esp_now_init() == ESP_OK) Serial.println("ESPNow Init Success!");
      else {
        Serial.println("ESPNow Init Failed....");
        ok = false;
      }
      Serial.print("Added Peer:");
      for (uint8_t i = 0; i < now_maxSlaves; i++) {
        now_nodeMAC[1] = i;
        memcpy(&slave.peer_addr, &now_nodeMAC, 6);
        slave.channel = now_Channel;
        slave.encrypt = 0;
        if( esp_now_add_peer(peer) == ESP_OK) Serial.printf("%u%s", i, (i < (now_maxSlaves - 1) ? ", " : "\n"));
      }
      esp_now_register_recv_cb(now_onDataRecv);
      esp_now_register_send_cb(now_onDataSent);
      return ok;
    }

    void now_sendData(uint8_t _mac, const char *_msg) {
        now_nodeMAC[0] = 0;
        now_nodeMAC[1] = _mac;
        if( esp_now_send(now_nodeMAC, (uint8_t*)_msg, strlen(_msg) + 1) == ESP_OK) Serial.printf("Success Sent msg: %s\n", _msg);
        else Serial.printf("DID NOT SEND msg: %s\n", _msg);
    }

    void now_onDataRecv(const uint8_t *mac, const uint8_t *data, int len) {
      char _msg[len + 2];  
      snprintf(_msg, len + 1, "%s", (char*)data);
      Serial.printf("Received: %d Bytes, msg: %s\n", len, _msg);
    }
    
    void now_onDataSent(const uint8_t *mac_addr, esp_now_send_status_t status) {
      char macStr[18];
      snprintf(macStr, sizeof(macStr), "%02x:%02x:%02x:%02x:%02x:%02x",
               mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]);
      Serial.print("Last Packet Sent to: "); Serial.println(macStr);
      Serial.print("Last Packet Send Status: "); Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Delivery Success" : "Delivery Fail");
    }
  
  #endif
#endif
