/************* ESP MLX90614 temperature *************
  ***************************************************
  * Ewald Kendziorra sr, 2016, http://kendziorra.nl *
  * License:        Free                            *
  * Publishing AFP: Only with this header file      *
  * Editing:        Free                            *
  * Guarantee:      No                              *
  * Using:          At your own risk                *
  * Support:        No                              *
  ***************************************************/

/** MLX90614: High precision non-contact temperature measurements
    -40 to 125 ˚C for sensor temperature
    -70 to 380 ˚C for object temperature
**/

#ifndef _MLX90614_
#define _MLX90614_
#include <Wire.h>

enum MLX {  // address and registers
  mlxAddress = 0x5A,
  mlxAmbient = 0x06,
  mlxObject = 0x07
};

float sGetMlxTemp(MLX reg) {
  Wire.beginTransmission(mlxAddress);
  Wire.write(reg);
  Wire.endTransmission(false);
  Wire.requestFrom(mlxAddress, 3);
  uint16_t k = Wire.read();
  k |= Wire.read() << 8;
  uint16_t n = Wire.read();
  if (reg == mlxAmbient) n = 0;
  else n = 1;
  /** interpolate and extrapolate temperatures correction **/
  return k * 0.02 + (mlxCalibrate[n][1] - mlxCalibrate[n][0]) / 100 + mlxCalibrate[n][0];
}

#endif
