/** kendziorra.nl, june 2020, totally free to use **/
/** api.openweathermap.org current weather 
    Useful link: API documentation https://openweathermap.org/api
  name         : char array     units (standard 'metric' optianal 'imperial')
  ----------------------------------------
  timeZone     : owm.zone       sec
  sunRise      : owm.rise[0-2], hh:mm:ss
  sunSet       : owm.set[0-2],  hh:mm:ss
  temperature  : owm.temp       *C
  humidity     : owm.hum        %
  pressure     : owm.pres       hPa
  windSpeed    : owm.wind       m/s
  windDirection: owm.dir        abbreviation N.., E.., S.., W..
  **/

#ifndef _S_OPENWEATHERMAP_H_
  #define _S_OPENWEATHERMAP_H_
  
  #include <HTTPClient.h>
  
  char url[150] = "http://api.openweathermap.org/data/2.5/weather?";
  const char itemStart[8][14] = {"<timezone>", " rise=\"", " set=\"", "<temperature ", "<humidity ", "<pressure ", "<speed ", "<direction "},
             itemEnd[8][9]  = {"</", "T", "T", "value=\"", "value=\"", "value=\"", "value=\"", "code=\""};
  struct OWM {
    char zone[8];
    char rise[3][3];
    char set[3][3];
    char temp[8];
    char hum[4];
    char pres[5];
    char wind[7];
    char dir[4];
  } owm;
  bool showInfo = false;  // info to monitor
  
  void weatherInit(char *_lat, char *_lon, char *_units, char *_key, bool _print = showInfo);
  bool getWeather();
  void getOwm(String str);
  void parseOwm(String str, uint16_t pos, uint8_t p, uint8_t n, char * item);
  
  void weatherInit(char *_lat, char *_lon, char *_units, char *_key, bool _print) {
    showInfo = _print;
    sprintf(url + strlen(url), "lat=%s&lon=%s&mode=xml&units=%s&APPID=%s", _lat, _lon, _units, _key);
    if (showInfo) Serial.println(url);
  }
  
  bool getWeather() {
    bool f = true;
    if (!WiFi.isConnected()) {
      WiFi.begin();
      delay(500);
    }
    HTTPClient http;
    http.begin(url);
    int httpCode = http.GET();
    if (httpCode > 0) getOwm(http.getString());
    else f = false;
    http.end();
    return f;
  }    
  
  void getOwm(String str) {
    if (showInfo) Serial.println(str);
    int16_t p1[8], p2, p3;
    for (uint8_t i = 0; i < 8; i++) {
      int16_t n = 0;
      p1[i] = str.indexOf(itemStart[i] + n);
      if (p1[i] > 0) {
        p1[i] += strlen(itemStart[i]);
        n +=  p1[i];
      }
    }
    if (p1[0] > 0) {
      p2 = str.indexOf(itemEnd[0], p1[0]);
      strcpy(owm.zone, str.substring(p1[0], p2).c_str());
    }
    if (p1[1] > 0) {  
      p2 = str.indexOf(itemEnd[1], p1[1]) + 1;
      strcpy(owm.rise[0], str.substring(p2    , p2 + 2).c_str());
      strcpy(owm.rise[1], str.substring(p2 + 3, p2 + 5).c_str());
      strcpy(owm.rise[2], str.substring(p2 + 6, p2 + 8).c_str());
    }
    if (p1[2] > 0) {  
      p2 = str.indexOf(itemEnd[2], p1[2]) + 1;
      strcpy(owm.set[0], str.substring(p2    , p2 + 2).c_str());
      strcpy(owm.set[1], str.substring(p2 + 3, p2 + 5).c_str());
      strcpy(owm.set[2], str.substring(p2 + 6, p2 + 8).c_str());
    }
    parseOwm(str, p1[3], 3, 7, owm.temp);
    parseOwm(str, p1[4], 4, 7, owm.hum);
    parseOwm(str, p1[5], 5, 7, owm.pres);
    parseOwm(str, p1[6], 6, 7, owm.wind);
    parseOwm(str, p1[7], 7, 6, owm.dir);
  }
   
  void parseOwm(String str, uint16_t pos, uint8_t p, uint8_t n, char * item) {
    int16_t p2 = str.indexOf(itemEnd[p], pos) + n;
    int16_t p3 = str.indexOf("\"", p2);
    strcpy(item, str.substring(p2, p3).c_str());
  }

#endif