
#ifndef SIMPLESTEPPER_H
#define SIMPLESTEPPER_H
    
uint16_t _stepPos = 0,  // stepper position
         _endPos = 0;   // last step position (HOME to END position in 512 or 4096 steps)
void sGotoHome();
void sGotoEnd();
void stepper(int16_t);  // 0- 511 = 360* or 0-4095 = 360*
void stepTo(uint16_t);  // go to step nn (from HOME), 512 or 4096 steps is 360*
void smallSteps(int16_t);  // 0-4095 = 360*
void setSteps(const uint8_t *);
void findHomeEnd(bool toHome);
const uint8_t steps[9][4] = {  // 8 steps and one default (steps[8])
                {0, 0, 0, 1}, {0, 0, 1, 1}, {0, 0, 1, 0}, {0, 1, 1, 0},
                {0, 1, 0, 0}, {1, 1, 0, 0}, {1, 0, 0, 0}, {1, 0, 0, 1},
  /*default ->*/{0, 0, 0, 0}};/*<- default*/
uint8_t in1234[4],  // stepper pins 
        _homePin,    // microswitch home position, pin with pull-up
        _endPin,     // microswitch end position, pin with pull-up
        stepSpeed;  // stepper speed, delay in ms,  default 1, min: 1, max: 50
uint16_t maxRotations = 5;  // The maximun number of rotations to find the home/end position
bool _isHome,
        _isEnd,
        rotSteps;  // false: 512 steps is 360*, true: 4096 steps is 360*

void sStepsInit(uint8_t _stepPin1, uint8_t _stepPin2, uint8_t _stepPin3, uint8_t _stepPin4, uint8_t _homePin, int16_t _endPin, bool _rotSteps, uint8_t _stepSpeed, uint16_t _maxRotations);

void sStepsInit(uint8_t _stepPin1, uint8_t _stepPin2, uint8_t _stepPin3, uint8_t _stepPin4, uint8_t _homePin, int16_t _endPin, bool _rotSteps, uint8_t _stepSpeed, uint16_t _maxRotations) {
  in1234[0] = _stepPin1;
  in1234[1] = _stepPin2;
  in1234[2] = _stepPin3;
  in1234[3] = _stepPin4;
  _homePin = _homePin;
  _endPin = _endPin;
  rotSteps = _rotSteps;
  stepSpeed = _stepSpeed;
  maxRotations = _maxRotations;
  
  for (uint8_t i = 0; i <= 3; i++) pinMode(in1234[i], OUTPUT);
  pinMode(_homePin, INPUT_PULLUP);
  sGotoHome();  // FIRST: find the HOME position 
  if (_endPin >= 0) {
    pinMode(_endPin, INPUT_PULLUP);
    sGotoEnd();   // FIRST: find the END position 
    _endPos = _stepPos;
    sGotoHome();  // start point is position 0
  }
  else _endPos = abs(_endPin);  
}

/** find the home position **/
void sGotoHome() {  // turn LEFT
  findHomeEnd(true);
  _stepPos = 0;
}

/** find the end position **/
void sGotoEnd() {  // turn RIGHT
  if (_endPin >= 0) findHomeEnd(false);
  else stepTo(_endPos);
}

void findHomeEnd(bool toHome) {
  for (uint16_t r = 0; r < maxRotations; r++) {  // The maximun number of rotations to find the home/end position
    for (uint16_t i = 0; i <= 511; i++) {  // 512 steps is one 360* rotation
      if ((toHome ? _isHome : _isEnd)) return;
      stepper((toHome ? -1 : 1));  // turn left/right
    }
  }
}

/** turn to the requested position in <n> steps  **/
void stepTo(uint16_t nSteps) {
  int16_t n = -(_stepPos - nSteps);
  stepper(n);
}

/** do one stepper step **/
void setSteps(const uint8_t out[4]) {
  for (uint8_t i = 0; i <= 3; i++) digitalWrite(in1234[i], out[i]);
  if (digitalRead(_homePin) == LOW) {_isHome = true; _stepPos = 0;}
  else if (_endPin >= 0 && digitalRead(_endPin) == LOW) _isEnd = true;
  else if (_stepPos >= _endPos) _isEnd = true;
}

/** do <n> steps  **/
void stepper(int16_t nSteps) {
  if (nSteps == 0) return;
  if (rotSteps) {smallSteps(nSteps); return;}  // 0-4095 steps  = 360*
  // (0-511 steps = 360*)
  uint16_t stps = abs(nSteps);
  _isHome = _isEnd = false;
  if (nSteps > 0) {
    for (uint16_t n = 0; n < stps; n++)  // turn + (R)
      _stepPos++;
      for (uint8_t i = 0; i <= 7; i++) {
        setSteps(steps[i]);
        if (_isEnd) break;
        delay(stepSpeed);}
  }
  else {
    for (uint16_t n = 0; n < stps; n++)  // turn - (L)
      _stepPos--;
      for (int8_t i = 7; i >= 0; i--) {
        setSteps(steps[i]);
        if (_isHome) break;
        delay(stepSpeed);}
  }
  setSteps(steps[8]);  // stepper default values
}

/** turn to the requested position in <n> steps **/// (0-4095 steps = 360*)
void smallSteps(int16_t nSteps) {  // one nSteps = 1 stepper step, one rotation = 4096 nSteps
  uint16_t stps, s;
  _isHome = _isEnd = false;
  stps = s = abs(nSteps);
  if (nSteps > 0) {
    for (uint16_t n = 0; n < stps; n++) {  // turn + (R)
      for (uint8_t i = 0; i <= 7; i++) {
        _stepPos++;
        setSteps(steps[i]); 
        if (_isEnd || (--s == 0)) break;
        delay(stepSpeed);
      }
      if (_isEnd || (s == 0)) break;
    }
  }
  else {
    for (uint16_t n = 0; n < stps; n++) {  // turn - (L)
      for (int8_t i = 7; i >= 0; i--) {
        _stepPos--;
        setSteps(steps[i]); 
        if (_isHome ||  (--s == 0)) break;
        delay(stepSpeed);
      }
      if (_isHome || (s == 0)) break;
    }
  }
  setSteps(steps[8]);  // stepper default values
}

#endif
