#ifndef _S_THERMISTOR_H_
  #define _S_THERMISTOR_H_

    
  float getTemp() {
    /** ESP32 ADC chanels/analogread are not stable so we use buffers to smooth out the result **/ 
    static bool isThermInit = false;
    static double A, B, C;
    const uint8_t bufLen = 16;  // minimal: 16
    uint32_t adcBuf[bufLen], adc = 0;
    if (!isThermInit) {
      // calc Steinhart-Hart ABC https://en.wikipedia.org/wiki/Steinhart%E2%80%93Hart_equation#Inverse_of_the_equation
      double
        L1 = log((double)ntcR[0] * calibr),
        L2 = log((double)ntcR[1] * calibr),
        L3 = log((double)ntcR[2] * calibr),
        Y1 = 1.0 / ((double)ntcT[0] + 273.15),
        Y2 = 1.0 / ((double)ntcT[1] + 273.15),
        Y3 = 1.0 / ((double)ntcT[2] + 273.15),
        y2 = (Y2 - Y1) / (L2 - L1),
        y3 = (Y3 - Y1) / (L3 - L1);
      C = ((y3 - y2) / (L3 - L2)) * pow((L1 + L2 + L3), -1);
      B = y2 - C * (pow(L1, 2) + L1 * L2 + pow(L2, 2));
      A = Y1 - (B + pow(L1, 2) * C) * L1;
      isThermInit = true;
    }
    for (uint8_t i = 0; i < bufLen; i++) {
      adcBuf[i] = analogRead(tempPin);
      delay(15);
    }
    for (uint8_t i = 0; i < bufLen - 1; i++) {  // a simple bubble sort
      for (uint8_t j = 0; j < bufLen - i - 1; j++) {
        if (adcBuf[j] > adcBuf[j + 1]) {
          uint64_t v = adcBuf[j];
          adcBuf[j] = adcBuf[j + 1];
          adcBuf[j + 1] = v;
        }
      }
    }
    for (uint8_t i = 3; i < bufLen - 3; ++i) adc += adcBuf[i];  // strip 3-lowest and 3-highest value
    adc /= (bufLen - 6);
    double R = (double)serR * (4095.0 / (double)adc - 1.0),
           L = log(R),
           T = 1.0 / (A + B * L + C * pow(L, 3));
    #ifdef PRINT_RESISTANCE
      Serial.printf("resistance: %u", (int)R);  //##### get the resistance at a given temperature #####
    #endif  
    return (float)T - 273.15;
  }
  
#endif
