/** simpleSocketClient - adapted for Arduino ESP32 by Ewald Kendziorra sr, 2023
  *   https://github.com/espressif/esp-idf/blob/v4.3/examples/protocols/websocket/main/websocket_example.c
  * esp32 Websocket Server: "simpleSocketServer.h" 
 **/
#ifndef _SIMPLE_WS_CLIENT_
#define _SIMPLE_WS_CLIENT_
  
  #include <WiFi.h>
  #include <esp_websocket_client.h>  // (ESP32)
  
  bool ws_Connected = false;
  bool ws_Val = false;
  String ws_Buf = "";  // we use a String so do not suffer from a character array length problem
  
  esp_websocket_client_handle_t client;
  
  void onWs_Event(void *handler_args, esp_event_base_t base, int32_t event_id, void *event_data);
  void ws_Stop();
  void ws_Start();
  bool ws_Send(String str);
  
  void onWs_Event(void *handler_args, esp_event_base_t base, int32_t event_id, void *event_data) {
    if (event_id == WEBSOCKET_EVENT_CONNECTED) ws_Connected = true;
    else if (event_id == WEBSOCKET_EVENT_DISCONNECTED) ws_Connected = false;
    else if (event_id == WEBSOCKET_EVENT_DATA) {
      esp_websocket_event_data_t *data = (esp_websocket_event_data_t *)event_data;
      if (data->op_code != 1 || data->data_len == 0) return;  // only text 
      char c[data->data_len + 1];
      strncpy(c, ((char *)(data->data_ptr + data->payload_offset)), data->data_len);
      c[data->data_len] = '\0';
      ws_Buf = String(c);
      ws_Val = true;
    }
  }
  
  void ws_Stop() {  // Stop websocket client connection
    esp_websocket_client_stop(client);
    esp_websocket_client_destroy(client);
    ws_Connected = false;
  }
  
  void ws_Start() {  // Start websocket client connection
    esp_websocket_client_config_t websocket_cfg = {};
    websocket_cfg.uri = ws_URI;
    client = esp_websocket_client_init(&websocket_cfg);
    esp_websocket_register_events(client, WEBSOCKET_EVENT_ANY, onWs_Event, (void *)client);
    esp_websocket_client_start(client);
    delay(2000);
  }
  
  bool ws_Send(String str) {
    if (!ws_Connected) return false;
    uint16_t len = str.length() + 1;
    char data[len];
    strcpy(data, str.c_str());
    esp_websocket_client_send_text(client, data, len, portMAX_DELAY);
    return true;
  }

#endif
