/** simpleSocketServer - adapted for Arduino ESP32 by Ewald Kendziorra sr, 2023 
  * totally free to use and modify
  * this is a TEXT-only SYNC Websocket Server and use only the standard ESP32 esp_http_server.h library
  * ASYNC has problems with rapid successive actions and forget to send some data, so i deleted it
  * esp32 Websocket Clients: "simpleSocketClient.h" and/or the HTML-example
 **/
#ifndef _SIMPLE_WS_SERVER_
#define _SIMPLE_WS_SERVER_

  #include <WiFi.h>
  #include <esp_http_server.h>  // (ESP32)

  /** useful helpers **/ 
  size_t fds = 0;            // number of connectec clients (after call wsCliendList())
  int client_fds[16] = {0};  // connectec clients ID list   (after call wsCliendList())
  int clientNr = 0;     // current client ID
  /** use this in loop() **/
  bool wsVal = false;        // true: there is a new message (set it to false after msg pick up)
  String wsMsg = "";         // if wsVal: then there is a new message
  // used as default if not declared first, is changed after using a declared value
  String wsUri = "\ws";      
  uint16_t wsPort = 80;

  httpd_ws_frame_t ws_pkt;
  httpd_handle_t wsServer = NULL;

  uint8_t wsCliendList();
  bool wsCliendAvtive(int16_t wsClient);
  bool wsSendMsg(String msg, int fd = clientNr);
  esp_err_t wsHandler(httpd_req_t* req);
  bool start_wsServer();

  /**
          in "start_wsServer(...,..) you can change values after CONFIG
  #define HTTPD_DEFAULT_CONFIG() {                        \
          .task_priority      = tskIDLE_PRIORITY+5,       \
          .stack_size         = 4096,                     \
          .core_id            = tskNO_AFFINITY,           \
          .server_port        = 80,                       \
          .ctrl_port          = 32768,                    \
          .max_open_sockets   = 7,                        \
          .max_uri_handlers   = 8,                        \
          .max_resp_headers   = 8,                        \
          .backlog_conn       = 5,                        \
          .lru_purge_enable   = false,                    \
          .recv_wait_timeout  = 5,                        \
          .send_wait_timeout  = 5,                        \
          .global_user_ctx = NULL,                        \
          .global_user_ctx_free_fn = NULL,                \
          .global_transport_ctx = NULL,                   \
          .global_transport_ctx_free_fn = NULL,           \
          .enable_so_linger = false,                      \
          .linger_timeout = 0,                            \
          .open_fn = NULL,                                \
          .close_fn = NULL,                               \
          .uri_match_fn = NULL                            \
  }
  typedef enum {
      HTTPD_WS_TYPE_CONTINUE   = 0x0,
      HTTPD_WS_TYPE_TEXT       = 0x1,  // in my "simpleSocketServer.h": only this type is used...
      HTTPD_WS_TYPE_BINARY     = 0x2,  // ... text is sent as bytes so you can also send other data in TEXT-mode 
      HTTPD_WS_TYPE_CLOSE      = 0x8,
      HTTPD_WS_TYPE_PING       = 0x9,
      HTTPD_WS_TYPE_PONG       = 0xA
  } httpd_ws_type_t;
  typedef enum {
      HTTPD_WS_CLIENT_INVALID        = 0x0,
      HTTPD_WS_CLIENT_HTTP           = 0x1,
      HTTPD_WS_CLIENT_WEBSOCKET      = 0x2,
  } httpd_ws_client_info_t;

  **/

  uint8_t wsCliendList() {
    for (uint8_t i = 0; i < (size_t)CONFIG_LWIP_MAX_LISTENING_TCP; i++) client_fds[i] = 0;
    fds = (size_t)CONFIG_LWIP_MAX_LISTENING_TCP;
    esp_err_t ret = httpd_get_client_list(wsServer, &fds, client_fds);
    if (ret != ESP_OK) fds = 0;
    return (uint8_t) fds;
  }

  bool wsCliendAvtive(int wsClient) {
    if (httpd_ws_get_fd_info(wsServer, wsClient) == HTTPD_WS_CLIENT_WEBSOCKET) return true;
    else return false;
  }

  bool wsSendMsg(String msg, int fd) {
    bool ret = true;
    uint16_t len = msg.length() + 1;
    ws_pkt.type = HTTPD_WS_TYPE_TEXT; 
    ws_pkt.payload = (uint8_t*)msg.c_str();
    ws_pkt.len = len;
    if (fd == -1) {
      uint8_t n = wsCliendList();
      if (n > 0) {
        for (int i = 0; i < n; i++) {
          if (httpd_ws_send_data(wsServer, client_fds[i], &ws_pkt) != ESP_OK) ret = false;
        }
      }
    }
    else if (wsCliendAvtive(fd)) 
      if (httpd_ws_send_data(wsServer, fd, &ws_pkt) != ESP_OK) ret = false;
    return ret;
  }

  esp_err_t wsHandler(httpd_req_t* req) {
    if (req->method == HTTP_GET) return ESP_OK;
    clientNr = httpd_req_to_sockfd(req);
    ws_pkt.type = HTTPD_WS_TYPE_TEXT;
    ws_pkt.payload = {0};
    ws_pkt.len = 0;
    esp_err_t ret = httpd_ws_recv_frame(req, &ws_pkt, 0);  // get ws_pkt.len
    if (ret != ESP_OK) return ret;
    if (ws_pkt.len) {  // allocate space for pkt->payload
      uint8_t *buf = (uint8_t*)calloc(1, ws_pkt.len + 1);
      ws_pkt.payload = buf;
      httpd_ws_recv_frame(req, &ws_pkt, ws_pkt.len);  // get ws_pkt.payload
      wsMsg = String((char*)ws_pkt.payload);
      if (wsMsg.length() == ws_pkt.len) wsVal = true; // message OK for user
      else wsVal = false;
    }
    return ESP_OK;
  }

  bool start_wsServer() {
    ws_pkt.final = false;
    ws_pkt.fragmented = false;
    httpd_config_t config = HTTPD_DEFAULT_CONFIG();
    #ifdef WS_PORT
      config.server_port = WS_PORT;
      wsPort = config.server_port;
    #else
      config.server_port = wsPort;
    #endif
    wsPort = config.server_port;
    wsServer = NULL;
    if (httpd_start(&wsServer, &config) == ESP_OK) {
      #ifdef WS_URI
        wsUri = WS_URI;
      #endif
      httpd_uri_t ws = {
        .uri = wsUri.c_str(),
        .method = HTTP_GET,
        .handler = wsHandler,
        .user_ctx = NULL,
        .is_websocket = true
      };
      httpd_register_uri_handler(wsServer, &ws);
    }
    if (wsServer == NULL) return false;
    return true;
  }

#endif
